
# Copyright(c) Microsoft Corporation. All rights reserved.
#
# Run tests using both RxInSqlServer and local compute contexts and compare results
# See SqlServerIOQ_SetupAndRun.R for setup information
#
############################################################################################


#############################################################################################
# Function to get compute contexts
#############################################################################################

"getSqlServerComputeContext" <- function()
{
    rxTestArgs <- getOption("rxTestArgs")
    if (is.null(rxTestArgs))
    {
        stop("Compute context information must be in 'rxTestArgs' option.  See SqlServerIOQ_SetupAndRun.R.")
    }

    testCluster <- RxInSqlServer(
        connectionString = rxTestArgs$connectionString,
        numTasks = rxTestArgs$numTasks,
        #shareDir = rxTestArgs$shareDir,
        #remoteShareDir = rxTestArgs$remoteShareDir,
        revoPath = rxTestArgs$revoPath,
        consoleOutput = rxTestArgs$consoleOutput,
        wait = rxTestArgs$wait,
        autoCleanup = rxTestArgs$autoCleanup,
        traceEnabled = rxTestArgs$traceEnabled,
        traceLevel = rxTestArgs$traceLevel
    )

    return(testCluster)
}

"getLocalComputeContext" <- function()
{
    RxLocalSeq()
}

#############################################################################################
# Functions to get data sources
#############################################################################################
"getAirDemoSqlServerDS" <- function(specifyLevels = TRUE)
{
    rxTestArgs <- getOption("rxTestArgs")
    if (is.null(rxTestArgs))
    {
        stop("Compute context information must be in 'rxTestArgs' option.  See SqlServerIOQ_SetupAndRun.R.")
    }

    sqlConnString = rxTestArgs$connectionString
    airColInfo <- NULL
    if (specifyLevels == TRUE)
    {
        airColInfo <- list(
            ArrDelay = list(type = "integer", low = -86, high = 1490),
            CRSDepTime = list(low = .016667, high = 23.98333),
            DayOfWeek = list(type = "factor",
                levels = c("Monday", "Tuesday", "Wednesday", "Thursday",
                "Friday", "Saturday", "Sunday")))
    }
    else
        {
        airColInfo <- list(
            ArrDelay = list(type = "integer"),
            DayOfWeek = list(type = "factor"),
            CRSDepTime = list(type = "float32"))
    }


    testDataSource <- RxSqlServerData(
        connectionString = sqlConnString,
        table = "AirlineDemoSmall",
        colInfo = airColInfo,
        rowsPerRead = 5000)

    return(testDataSource)

}

"getAirDemoXdfLocalDS" <- function()
{
    RxXdfData(file = file.path(rxGetOption("sampleDataDir"), "AirlineDemoSmall.xdf"),
              fileSystem = "native")
}

"getClaimsSqlServerDS" <- function(specifyLevels = TRUE)
{
    rxTestArgs <- getOption("rxTestArgs")
    if (is.null(rxTestArgs))
    {
        stop("Compute context information must be in 'rxTestArgs' option.  See SqlServerIOQ_SetupAndRun.R.")
    }

    sqlConnString <- rxTestArgs$connectionString
    dataColInfo <- NULL
    if (isTRUE(specifyLevels))
    {
        dataColInfo <- list(
            RowNum = list(type = "integer"),
            age = list(type = "factor",
                levels = c("17-20", "21-24", "25-29", "30-34", "35-39", "40-49", "50-59", "60+")),
            car.age = list(type = "factor",
                levels = c("0-3", "4-7", "8-9", "10+")),
            type = list(type = "factor",
                levels = c("A", "B", "C", "D")),
            cost = list(type = "float32"),
            number = list(type = "float32"))
    }
    else
        {
        dataColInfo <- list(
            RowNum = list(type = "integer"),
            age = list(type = "factor"),
            car.age = list(type = "factor"),
            type = list(type = "factor"),
            cost = list(type = "float32"),
            number = list(type = "float32"))
    }

    testDataSource <- RxSqlServerData(
        connectionString = sqlConnString,
        table = "claims",
        colInfo = dataColInfo,
        rowsPerRead = 5000)

    return(testDataSource)
}

"getClaimsXdfLocalDS" <- function()
{
    RxXdfData(file = file.path(rxGetOption("sampleDataDir"), "claims4blocks.xdf"),
              fileSystem = "native")
}

"getTestOdbc" <- function()
{
    rxTestArgs <- getOption("rxTestArgs")
    if (!is.null(rxTestArgs) && !is.null(rxTestArgs$rxTestOdbc) && (rxTestArgs$rxTestOdbc == TRUE))
    {
        return(TRUE)
    }
    return(FALSE)
}

"getHaveWritePermission" <- function()
{
    rxTestArgs <- getOption("rxTestArgs")
    if (!is.null(rxTestArgs) && !is.null(rxTestArgs$rxHaveWritePermission) &&
        (rxTestArgs$rxHaveWritePermission == FALSE))
    {
        return(FALSE)
    }
    return(TRUE)
}

"rxuTestAnalysisFunctionWithAirDemo" <- function(analysisName, analysisParams, testValues,
    printFirstOutput = FALSE)
{
    origComputeContext <- rxGetComputeContext()
    on.exit(rxSetComputeContext(origComputeContext), add = TRUE)

    localComputeContext <- getLocalComputeContext()
    distributedComputeContext <- getSqlServerComputeContext()
    localTestDataSource <- getAirDemoXdfLocalDS()
    sqlServerTestDataSource <- getAirDemoSqlServerDS()

    rxSetComputeContext(localComputeContext)

    analysisParams$data <- localTestDataSource
    localResult <- do.call(analysisName, analysisParams)
    if (printFirstOutput)
    {
        cat("\nLocal results are:\n")
        print(localResult)
    }
    else
        {
        out1 <- capture.output(print(localResult))
        cat("\n", out1[1], "\n")
        cat("\n", out1[2], "\n")
    }

    testOdbc <- getTestOdbc()
    if (testOdbc)
    {
        testDataSources <- list(sqlServerTestDataSource, sqlServerTestDataSource)
    }
    else
        {
        testDataSources <- list(sqlServerTestDataSource)
    }


    for (i in 1:length(testDataSources))
    {
        thisDataSource <- testDataSources[[i]]
        cat("-----------------------------------------------------------------------------\n")
        if (!testOdbc || (i == 2))
        {
            cat(paste("Testing RxInSqlServer compute context\n"))
            rxSetComputeContext(distributedComputeContext)
            SqlServerType <- "InSqlServer"
        }
        else
            {
            cat(paste("Testing RxSqlServerData data source in local compute context\n"))
            SqlServerType <- "ODBC"
        }

        cat("\n")
        analysisParams$data <- testDataSources[[i]]
        tm <- system.time(
            currentResult <- do.call(analysisName, analysisParams)
            )
        cat("Elapsed time for analysis:", tm["elapsed"], "\n")
        thisTime <- as.numeric(tm["elapsed"])
        if (!exists("rxSqlServerTimings"))
        {
            rxSqlServerTimings <<- data.frame(Analysis = analysisName,
                Type = SqlServerType, Time = thisTime, stringsAsFactors = FALSE)
        }
        else
            {
            newRow <- nrow(rxSqlServerTimings) + 1
            rxSqlServerTimings[newRow, "Analysis"] <<- analysisName
            rxSqlServerTimings[newRow, "Type"] <<- SqlServerType
            rxSqlServerTimings[newRow, "Time"] <<- thisTime
        }

        #TODO (lgong) 2016-03-30: need to find a way to compare TLC models.
        if (is.null(testValues))
        {
            cat(paste("    Testing for equal output.\n"))
            checkEquals(localResult, currentResult)
        }
        else
            {
            for (j in 1:length(testValues))
            {
                currentTestValue <- currentResult[testValues[j]]
                cat(paste("    Testing output component:", testValues[j], "\n"))
                checkEquals(localResult[testValues[j]], currentTestValue)
            }
        }
    }
    rxSetComputeContext(localComputeContext)
}

"rxuEnsureSameClientServerCollate" <- function()
{
    # ensure that all the server nodes have the same collate.
    clusterCollate <- function(checkUnique = TRUE)
    {
        cCollate <- rxExec("Sys.getlocale", category = "LC_COLLATE")
        cCollate <- unlist(cCollate)

        if (checkUnique)
        {
            cCollate <- unique(cCollate)
            if (length(cCollate) != 1)
            {
                stop("LC_COLLATE is not the same for the nodes in current cluster context")
            }
        }

        return(cCollate)
    }

    origComputeContext <- rxGetComputeContext()
    on.exit(rxSetComputeContext(origComputeContext), add = TRUE)
    distributedComputeContext <- getSqlServerComputeContext()
    rxSetComputeContext(distributedComputeContext)

    # make sure LC_COLLATE is the same for both client and server,
    # which could affect sort() and how factor levels are ordered. 
    cCollate <- clusterCollate()
    lCollate <- Sys.getlocale("LC_COLLATE")
    if (lCollate != cCollate)
    {
        warning("changing the client collate (", lCollate, ") to the server collate (", cCollate, ").")
        Sys.setlocale("LC_COLLATE", cCollate)
    }
}

#rxuEnsureSameClientServerCollate()
"test.AAA.SqlServer" <- function()
{
    sqlCompute <- getSqlServerComputeContext()
    cat(paste("Connection String Tested:\n",
        "  ", sqlCompute@connectionString, "\n"))
    rxuEnsureSameClientServerCollate()
}

#############################################################################################
# Core High-Performance Analytics Algorithms Available In-Database
#############################################################################################

"test.SqlServer.HPA.rxFastForest_binary" <- function()
{
    type <- "binary"
    form <- age30 ~ car.age + type + cost + number
    transforms <- quote(list(age30 = (age == "30-34" | age == "35-39")))
    rowSelection <- quote(number > 10)
    writeModelVars <- FALSE
    extraVarsToWrite <- c("RowNum", "age", "age30")

    # Test code for local computations
    system.time(
    {
        rxSetComputeContext(getLocalComputeContext())
        local.data <- getClaimsXdfLocalDS()
        local.model <- rxFastForest(form, data = local.data, type = type,
            transforms = transforms, rowSelection = rowSelection
            )
    })
    local.summ <- summary(local.model)
    local.score <- rxPredict(local.model, data = local.data,
        writeModelVars = writeModelVars, extraVarsToWrite = extraVarsToWrite)
    checkEquals(extraVarsToWrite, colnames(local.score)[seq_along(extraVarsToWrite)])

    # Test code for in-database computations
    system.time(
    {
        rxSetComputeContext(getSqlServerComputeContext())
        sqlserver.data <- getClaimsSqlServerDS()
        sqlserver.model <- rxFastForest(form, data = sqlserver.data, type = type,
            transforms = transforms, rowSelection = rowSelection)
    })
    sqlserver.summ <- summary(sqlserver.model)
    sqlserver.score <- rxPredict(sqlserver.model, data = sqlserver.data,
        writeModelVars = writeModelVars, extraVarsToWrite = extraVarsToWrite)
    checkEquals(extraVarsToWrite, colnames(sqlserver.score)[seq_along(extraVarsToWrite)])

    # compare local and sqlserver
    #TODO (lgong) 2016-03-30: why outModelBin do not match?
    domatch <- c("Formula", "mamlCode", "Description", "summaryType")
    #dontmatch <- c("dataSource", "params", "call", "outModel", "outModelBin")
    checkEquals(local.model[domatch], sqlserver.model[domatch])

    checkEquals(coef(local.summ), coef(sqlserver.summ))
    checkEquals(local.score, sqlserver.score)
}

"test.SqlServer.HPA.rxFastForest_regression" <- function()
{
    type <- "regression"
    form <- cost10 ~ car.age + type + age + number
    transforms <- quote(list(cost10 = 10 * cost))
    rowSelection <- quote(number > 10)
    writeModelVars <- FALSE
    extraVarsToWrite <- c("RowNum", "cost", "cost10")

    # Test code for local computations
    system.time(
    {
        rxSetComputeContext(getLocalComputeContext())
        local.data <- getClaimsXdfLocalDS()
        local.model <- rxFastForest(form, data = local.data, type = type,
            transforms = transforms, rowSelection = rowSelection)
    })
    local.summ <- summary(local.model)
    local.score <- rxPredict(local.model, data = local.data,
        writeModelVars = writeModelVars, extraVarsToWrite = extraVarsToWrite)
    checkEquals(extraVarsToWrite, colnames(local.score)[seq_along(extraVarsToWrite)])

    # Test code for in-database computations
    system.time(
    {
        rxSetComputeContext(getSqlServerComputeContext())
        sqlserver.data <- getClaimsSqlServerDS()
        sqlserver.model <- rxFastForest(form, data = sqlserver.data, type = type,
            transforms = transforms, rowSelection = rowSelection)
    })
    sqlserver.summ <- summary(sqlserver.model)
    sqlserver.score <- rxPredict(sqlserver.model, data = sqlserver.data,
        writeModelVars = writeModelVars, extraVarsToWrite = extraVarsToWrite)
    checkEquals(extraVarsToWrite, colnames(sqlserver.score)[seq_along(extraVarsToWrite)])

    # compare local and sqlserver
    #TODO (lgong) 2016-03-30: why outModelBin do not match?
    domatch <- c("Formula", "mamlCode", "Description", "summaryType")
    #dontmatch <- c("dataSource", "params", "call", "outModel", "outModelBin")
    checkEquals(local.model[domatch], sqlserver.model[domatch])

    checkEquals(coef(local.summ), coef(sqlserver.summ))
    checkEquals(local.score, sqlserver.score)
}

"test.SqlServer.HPA.rxFastTrees_binary" <- function()
{
    type <- "binary"
    form <- age30 ~ car.age + type + cost + number
    transforms <- quote(list(age30 = (age == "30-34" | age == "35-39")))
    rowSelection <- quote(number > 10)
    writeModelVars <- FALSE
    extraVarsToWrite <- c("RowNum", "age", "age30")

    # Test code for local computations
    system.time(
    {
        rxSetComputeContext(getLocalComputeContext())
        local.data <- getClaimsXdfLocalDS()
        local.model <- rxFastTrees(form, data = local.data, type = type,
            transforms = transforms, rowSelection = rowSelection
            )
    })
    local.summ <- summary(local.model)
    local.score <- rxPredict(local.model, data = local.data,
        writeModelVars = writeModelVars, extraVarsToWrite = extraVarsToWrite)
    checkEquals(extraVarsToWrite, colnames(local.score)[seq_along(extraVarsToWrite)])

    # Test code for in-database computations
    system.time(
    {
        rxSetComputeContext(getSqlServerComputeContext())
        sqlserver.data <- getClaimsSqlServerDS()
        sqlserver.model <- rxFastTrees(form, data = sqlserver.data, type = type,
            transforms = transforms, rowSelection = rowSelection)
    })
    sqlserver.summ <- summary(sqlserver.model)
    sqlserver.score <- rxPredict(sqlserver.model, data = sqlserver.data,
        writeModelVars = writeModelVars, extraVarsToWrite = extraVarsToWrite)
    checkEquals(extraVarsToWrite, colnames(sqlserver.score)[seq_along(extraVarsToWrite)])

    # compare local and sqlserver
    #TODO (lgong) 2016-03-30: why outModelBin do not match?
    domatch <- c("Formula", "mamlCode", "Description", "summaryType")
    #dontmatch <- c("dataSource", "params", "call", "outModel", "outModelBin")
    checkEquals(local.model[domatch], sqlserver.model[domatch])

    checkEquals(coef(local.summ), coef(sqlserver.summ))
    checkEquals(local.score, sqlserver.score)
}

"test.SqlServer.HPA.rxFastTrees_regression" <- function()
{
    type <- "regression"
    form <- cost10 ~ car.age + type + age + number
    transforms <- quote(list(cost10 = 10 * cost))
    rowSelection <- quote(number > 10)
    writeModelVars <- FALSE
    extraVarsToWrite <- c("RowNum", "cost", "cost10")

    # Test code for local computations
    system.time(
    {
        rxSetComputeContext(getLocalComputeContext())
        local.data <- getClaimsXdfLocalDS()
        local.model <- rxFastTrees(form, data = local.data, type = type,
            transforms = transforms, rowSelection = rowSelection)
    })
    local.summ <- summary(local.model)
    local.score <- rxPredict(local.model, data = local.data,
        writeModelVars = writeModelVars, extraVarsToWrite = extraVarsToWrite)
    checkEquals(extraVarsToWrite, colnames(local.score)[seq_along(extraVarsToWrite)])

    # Test code for in-database computations
    system.time(
    {
        rxSetComputeContext(getSqlServerComputeContext())
        sqlserver.data <- getClaimsSqlServerDS()
        sqlserver.model <- rxFastTrees(form, data = sqlserver.data, type = type,
            transforms = transforms, rowSelection = rowSelection)
    })
    sqlserver.summ <- summary(sqlserver.model)
    sqlserver.score <- rxPredict(sqlserver.model, data = sqlserver.data,
        writeModelVars = writeModelVars, extraVarsToWrite = extraVarsToWrite)
    checkEquals(extraVarsToWrite, colnames(sqlserver.score)[seq_along(extraVarsToWrite)])

    # compare local and sqlserver
    #TODO (lgong) 2016-03-30: why outModelBin do not match?
    domatch <- c("Formula", "mamlCode", "Description", "summaryType")
    #dontmatch <- c("dataSource", "params", "call", "outModel", "outModelBin")
    checkEquals(local.model[domatch], sqlserver.model[domatch])

    checkEquals(coef(local.summ), coef(sqlserver.summ))
    checkEquals(local.score, sqlserver.score)
}

"test.SqlServer.HPA.rxEnsemble_regression" <- function()
{
    DEACTIVATED("this test seems to cause intermittent error on ShipR VM that cannot be reproduced locally.")

    type <- "regression"
    form <- cost10 ~ car.age + type + age + number
    transforms <- quote(list(cost10 = 10 * cost))
    rowSelection <- quote(number > 10)
    writeModelVars <- FALSE
    extraVarsToWrite <- c("RowNum", "cost", "cost10")
    randomSeed <- 123

    # Test code for local computations
    system.time(
    {
        rxSetComputeContext(getLocalComputeContext())
        local.data <- getClaimsXdfLocalDS()
        local.model <- rxEnsemble(form, data = local.data, type = type
            , transforms = transforms, rowSelection = rowSelection, randomSeed = randomSeed
            , trainers = list(fastTrees(), fastTrees(numTrees = 60), fastTrees(learningRate = 0.1))
            )
    })
    local.summ <- summary(local.model)
    local.score <- rxPredict(local.model, data = local.data,
        writeModelVars = writeModelVars, extraVarsToWrite = extraVarsToWrite)
    checkEquals(extraVarsToWrite, colnames(local.score)[seq_along(extraVarsToWrite)])

    # Test code for in-database computations
    system.time(
    {
        rxSetComputeContext(getSqlServerComputeContext())
        sqlserver.data <- getClaimsSqlServerDS()
        sqlserver.model <- rxEnsemble(form, data = sqlserver.data, type = type
            , transforms = transforms, rowSelection = rowSelection, randomSeed = randomSeed
            , trainers = list(fastTrees(), fastTrees(numTrees = 60), fastTrees(learningRate = 0.1))
            )
    })
    sqlserver.summ <- summary(sqlserver.model)
    sqlserver.score <- rxPredict(sqlserver.model, data = sqlserver.data,
        writeModelVars = writeModelVars, extraVarsToWrite = extraVarsToWrite)
    checkEquals(extraVarsToWrite, colnames(sqlserver.score)[seq_along(extraVarsToWrite)])

    # compare local and sqlserver
    #TODO (lgong) 2016-03-30: why outModelBin do not match?
    domatch <- c("Formula", "mamlCode", "Description", "summaryType")
    #dontmatch <- c("dataSource", "params", "call", "outModel", "outModelBin")
    checkEquals(local.model[domatch], sqlserver.model[domatch])

    checkEquals(coef(local.summ), coef(sqlserver.summ))
    checkEquals(local.score, sqlserver.score)
}

"test.SqlServer.HPA.rxLogisticRegression_binary" <- function()
{
    type <- "binary"
    form <- age30 ~ car.age + type + cost + number
    transforms <- quote(list(age30 = (age == "30-34" | age == "35-39")))
    rowSelection <- quote(number > 10)
    writeModelVars <- FALSE
    extraVarsToWrite <- c("RowNum", "age", "age30")

    # Test code for local computations
    system.time(
    {
        rxSetComputeContext(getLocalComputeContext())
        local.data <- getClaimsXdfLocalDS()
        local.model <- rxLogisticRegression(form, data = local.data, type = type,
            transforms = transforms, rowSelection = rowSelection
            )
    })
    local.summ <- summary(local.model)
    local.score <- rxPredict(local.model, data = local.data,
        writeModelVars = writeModelVars, extraVarsToWrite = extraVarsToWrite)
    checkEquals(extraVarsToWrite, colnames(local.score)[seq_along(extraVarsToWrite)])

    # Test code for in-database computations
    system.time(
    {
        rxSetComputeContext(getSqlServerComputeContext())
        sqlserver.data <- getClaimsSqlServerDS()
        sqlserver.model <- rxLogisticRegression(form, data = sqlserver.data, type = type,
            transforms = transforms, rowSelection = rowSelection)
    })
    sqlserver.summ <- summary(sqlserver.model)
    sqlserver.score <- rxPredict(sqlserver.model, data = sqlserver.data,
        writeModelVars = writeModelVars, extraVarsToWrite = extraVarsToWrite)
    checkEquals(extraVarsToWrite, colnames(sqlserver.score)[seq_along(extraVarsToWrite)])

    # compare local and sqlserver
    #TODO (lgong) 2016-03-30: why outModelBin do not match?
    domatch <- c("Formula", "mamlCode", "Description", "summaryType")
    #dontmatch <- c("dataSource", "params", "call", "outModel", "outModelBin")
    checkEquals(local.model[domatch], sqlserver.model[domatch])

    checkEquals(coef(local.summ), coef(sqlserver.summ))
    checkEquals(local.score, sqlserver.score)
}

"test.SqlServer.HPA.rxLogisticRegression_multiClass" <- function()
{
    type <- "multiClass"
    form <- type ~ age + car.age + cost10 + number
    transforms <- quote(list(cost10 = 10 * cost))
    rowSelection <- quote(number > 10)
    writeModelVars <- FALSE
    extraVarsToWrite <- c("RowNum", "type", "cost10")

    # Test code for local computations
    system.time(
    {
        rxSetComputeContext(getLocalComputeContext())
        local.data <- getClaimsXdfLocalDS()
        local.model <- rxLogisticRegression(form, data = local.data, type = type,
            transforms = transforms, rowSelection = rowSelection
            )
    })
    local.summ <- summary(local.model)
    local.score <- rxPredict(local.model, data = local.data,
        writeModelVars = writeModelVars, extraVarsToWrite = extraVarsToWrite)
    checkEquals(extraVarsToWrite, colnames(local.score)[seq_along(extraVarsToWrite)])

    # Test code for in-database computations
    system.time(
    {
        rxSetComputeContext(getSqlServerComputeContext())
        sqlserver.data <- getClaimsSqlServerDS()
        sqlserver.model <- rxLogisticRegression(form, data = sqlserver.data, type = type,
            transforms = transforms, rowSelection = rowSelection)
    })
    sqlserver.summ <- summary(sqlserver.model)
    sqlserver.score <- rxPredict(sqlserver.model, data = sqlserver.data,
        writeModelVars = writeModelVars, extraVarsToWrite = extraVarsToWrite)
    checkEquals(extraVarsToWrite, colnames(sqlserver.score)[seq_along(extraVarsToWrite)])

    # compare local and sqlserver
    #TODO (lgong) 2016-03-30: why outModelBin do not match?
    domatch <- c("Formula", "mamlCode", "Description", "summaryType")
    #dontmatch <- c("dataSource", "params", "call", "outModel", "outModelBin")
    checkEquals(local.model[domatch], sqlserver.model[domatch])

    checkEquals(coef(local.summ), coef(sqlserver.summ))
    checkEquals(local.score, sqlserver.score)
}

"test.SqlServer.HPA.SDCA_binary" <- function()
{
    type <- "binary"
    form <- age30 ~ car.age + type + cost + number
    transforms <- quote(list(age30 = (age == "30-34" | age == "35-39")))
    rowSelection <- quote(number > 10)
    writeModelVars <- FALSE
    extraVarsToWrite <- c("RowNum", "age", "age30")

    # Test code for local computations
    system.time(
    {
        rxSetComputeContext(getLocalComputeContext())
        local.data <- getClaimsXdfLocalDS()
        local.model <- rxFastLinear(form, data = local.data, type = type, trainThreads = 1,
            transforms = transforms, rowSelection = rowSelection, shuffle = FALSE)
    })
    local.summ <- summary(local.model)
    local.score <- rxPredict(local.model, data = local.data,
        writeModelVars = writeModelVars, extraVarsToWrite = extraVarsToWrite)
    checkEquals(extraVarsToWrite, colnames(local.score)[seq_along(extraVarsToWrite)])
    # Test code for in-database computations
    system.time(
    {
        rxSetComputeContext(getSqlServerComputeContext())
        sqlserver.data <- getClaimsSqlServerDS()
        sqlserver.model <- rxFastLinear(form, data = sqlserver.data, type = type, trainThreads = 1,
            transforms = transforms, rowSelection = rowSelection, shuffle = FALSE)
    })
    sqlserver.summ <- summary(sqlserver.model)
    sqlserver.score <- rxPredict(sqlserver.model, data = sqlserver.data,
        writeModelVars = writeModelVars, extraVarsToWrite = extraVarsToWrite)
    checkEquals(extraVarsToWrite, colnames(sqlserver.score)[seq_along(extraVarsToWrite)])

    # compare local and sqlserver
    #TODO (lgong) 2016-03-30: why outModelBin do not match?
    domatch <- c("Formula", "mamlCode", "Description", "summaryType")
    #dontmatch <- c("dataSource", "params", "call", "outModel", "outModelBin")
    checkEquals(local.model[domatch], sqlserver.model[domatch])
    checkEquals(coef(local.summ), coef(sqlserver.summ))
    checkEquals(local.score, sqlserver.score)
}

"test.SqlServer.HPA.SDCA_regression" <- function()
{
    type <- "regression"
    form <- cost10 ~ car.age + type + age + number
    transforms <- quote(list(cost10 = 10 * cost))
    rowSelection <- quote(number > 10)
    writeModelVars <- FALSE
    extraVarsToWrite <- c("RowNum", "cost", "cost10")

    # Test code for local computations
    system.time(
    {
        rxSetComputeContext(getLocalComputeContext())
        local.data <- getClaimsXdfLocalDS()
        local.model <- rxFastLinear(form, data = local.data, type = type, trainThreads = 1,
            transforms = transforms, rowSelection = rowSelection, shuffle = FALSE)
    })
    local.summ <- summary(local.model)
    local.score <- rxPredict(local.model, data = local.data,
        writeModelVars = writeModelVars, extraVarsToWrite = extraVarsToWrite)
    checkEquals(extraVarsToWrite, colnames(local.score)[seq_along(extraVarsToWrite)])

    # Test code for in-database computations
    system.time(
    {
        rxSetComputeContext(getSqlServerComputeContext())
        sqlserver.data <- getClaimsSqlServerDS()
        sqlserver.model <- rxFastLinear(form, data = sqlserver.data, type = type, trainThreads = 1,
            transforms = transforms, rowSelection = rowSelection, shuffle = FALSE)
    })
    sqlserver.summ <- summary(sqlserver.model)
    sqlserver.score <- rxPredict(sqlserver.model, data = sqlserver.data,
        writeModelVars = writeModelVars, extraVarsToWrite = extraVarsToWrite)
    checkEquals(extraVarsToWrite, colnames(sqlserver.score)[seq_along(extraVarsToWrite)])

    # compare local and sqlserver
    #TODO (lgong) 2016-03-30: why outModelBin do not match?
    domatch <- c("Formula", "mamlCode", "Description", "summaryType")
    #dontmatch <- c("dataSource", "params", "call", "outModel", "outModelBin")
    checkEquals(local.model[domatch], sqlserver.model[domatch])
    checkEquals(coef(local.summ), coef(sqlserver.summ))
    checkEquals(local.score, sqlserver.score)
}

"test.SqlServer.HPA.rxNeuralNet_binary" <- function()
{
    type <- "binary"
    form <- age30 ~ car.age + type + cost + number
    transforms <- quote(list(age30 = (age == "30-34" | age == "35-39")))
    rowSelection <- quote(number > 10)
    writeModelVars <- FALSE
    extraVarsToWrite <- c("RowNum", "age", "age30")

    # Test code for local computations
    system.time(
    {
        rxSetComputeContext(getLocalComputeContext())
        local.data <- getClaimsXdfLocalDS()
        local.model <- rxNeuralNet(form, data = local.data, type = type,
            transforms = transforms, rowSelection = rowSelection
            )
    })
    local.summ <- summary(local.model)
    local.score <- rxPredict(local.model, data = local.data,
        writeModelVars = writeModelVars, extraVarsToWrite = extraVarsToWrite)
    checkEquals(extraVarsToWrite, colnames(local.score)[seq_along(extraVarsToWrite)])

    # Test code for in-database computations
    system.time(
    {
        rxSetComputeContext(getSqlServerComputeContext())
        sqlserver.data <- getClaimsSqlServerDS()
        sqlserver.model <- rxNeuralNet(form, data = sqlserver.data, type = type,
            transforms = transforms, rowSelection = rowSelection)
    })
    sqlserver.summ <- summary(sqlserver.model)
    sqlserver.score <- rxPredict(sqlserver.model, data = sqlserver.data,
        writeModelVars = writeModelVars, extraVarsToWrite = extraVarsToWrite)
    checkEquals(extraVarsToWrite, colnames(sqlserver.score)[seq_along(extraVarsToWrite)])

    # compare local and sqlserver
    #TODO (lgong) 2016-03-30: why outModelBin do not match?
    domatch <- c("Formula", "mamlCode", "Description", "summaryType")
    #dontmatch <- c("dataSource", "params", "call", "outModel", "outModelBin")
    checkEquals(local.model[domatch], sqlserver.model[domatch])

    checkEquals(coef(local.summ), coef(sqlserver.summ))
    checkEquals(local.score, sqlserver.score)
}

"test.SqlServer.HPA.rxNeuralNet_multiClass" <- function()
{
    type <- "multiClass"
    form <- type ~ age + car.age + cost10 + number
    transforms <- quote(list(cost10 = 10 * cost))
    rowSelection <- quote(number > 10)
    writeModelVars <- FALSE
    extraVarsToWrite <- c("RowNum", "type", "cost10")

    # Test code for local computations
    system.time(
    {
        rxSetComputeContext(getLocalComputeContext())
        local.data <- getClaimsXdfLocalDS()
        local.model <- rxNeuralNet(form, data = local.data, type = type,
            transforms = transforms, rowSelection = rowSelection
            )
    })
    local.summ <- summary(local.model)
    local.score <- rxPredict(local.model, data = local.data,
        writeModelVars = writeModelVars, extraVarsToWrite = extraVarsToWrite)
    checkEquals(extraVarsToWrite, colnames(local.score)[seq_along(extraVarsToWrite)])

    # Test code for in-database computations
    system.time(
    {
        rxSetComputeContext(getSqlServerComputeContext())
        sqlserver.data <- getClaimsSqlServerDS()
        sqlserver.model <- rxNeuralNet(form, data = sqlserver.data, type = type,
            transforms = transforms, rowSelection = rowSelection)
    })
    sqlserver.summ <- summary(sqlserver.model)
    sqlserver.score <- rxPredict(sqlserver.model, data = sqlserver.data,
        writeModelVars = writeModelVars, extraVarsToWrite = extraVarsToWrite)
    checkEquals(extraVarsToWrite, colnames(sqlserver.score)[seq_along(extraVarsToWrite)])

    # compare local and sqlserver
    #TODO (lgong) 2016-03-30: why outModelBin do not match?
    domatch <- c("Formula", "mamlCode", "Description", "summaryType")
    #dontmatch <- c("dataSource", "params", "call", "outModel", "outModelBin")
    checkEquals(local.model[domatch], sqlserver.model[domatch])

    checkEquals(coef(local.summ), coef(sqlserver.summ))
    checkEquals(local.score, sqlserver.score)
}

"test.SqlServer.HPA.rxNeuralNet_regression" <- function()
{
    #DEACTIVATED("Defect 7281471: need to expose the maxNorm option for rxNeuralNet and document how to use it.")

    type <- "regression"
    form <- cost10 ~ car.age + type + age + number
    transforms <- quote(list(cost10 = 10 * cost))
    rowSelection <- quote(number > 10)
    writeModelVars <- FALSE
    extraVarsToWrite <- c("RowNum", "cost", "cost10")

    # Test code for local computations
    system.time(
    {
        rxSetComputeContext(getLocalComputeContext())
        local.data <- getClaimsXdfLocalDS()
        local.model <- rxNeuralNet(form, data = local.data, type = type,
            transforms = transforms, rowSelection = rowSelection)
    })
    local.summ <- summary(local.model)
    local.score <- rxPredict(local.model, data = local.data,
        writeModelVars = writeModelVars, extraVarsToWrite = extraVarsToWrite)
    checkEquals(extraVarsToWrite, colnames(local.score)[seq_along(extraVarsToWrite)])

    # Test code for in-database computations
    system.time(
    {
        rxSetComputeContext(getSqlServerComputeContext())
        sqlserver.data <- getClaimsSqlServerDS()
        sqlserver.model <- rxNeuralNet(form, data = sqlserver.data, type = type,
            transforms = transforms, rowSelection = rowSelection)
    })
    sqlserver.summ <- summary(sqlserver.model)
    sqlserver.score <- rxPredict(sqlserver.model, data = sqlserver.data,
        writeModelVars = writeModelVars, extraVarsToWrite = extraVarsToWrite)
    checkEquals(extraVarsToWrite, colnames(sqlserver.score)[seq_along(extraVarsToWrite)])

    # compare local and sqlserver
    #TODO (lgong) 2016-03-30: why outModelBin do not match?
    domatch <- c("Formula", "mamlCode", "Description", "summaryType")
    #dontmatch <- c("dataSource", "params", "call", "outModel", "outModelBin")
    checkEquals(local.model[domatch], sqlserver.model[domatch])

    checkEquals(coef(local.summ), coef(sqlserver.summ))
    checkEquals(local.score, sqlserver.score)
}

"test.SqlServer.HPA.rxOneClassSvm" <- function()
{
    #TODO (lgong) 2016-03-26: how to interrupt TLC computation	
    whichAge <- "60+"
    form <- agen ~ car.age + type + cost + number
    transforms <- bquote(list(agen = !(age == .(whichAge))))
    rowSelection <- bquote(!(age == .(whichAge)))
    writeModelVars <- FALSE
    extraVarsToWrite <- c("RowNum", "age", "agen")

    # Test code for local computations
    system.time(
    {
        rxSetComputeContext(getLocalComputeContext())
        local.data <- getClaimsXdfLocalDS()
        local.model <- rxOneClassSvm(form, data = local.data,
            transforms = transforms, rowSelection = rowSelection)
    })
    local.summ <- summary(local.model)
    local.score <- rxPredict(local.model, data = local.data,
        writeModelVars = writeModelVars, extraVarsToWrite = extraVarsToWrite)
    checkEquals(extraVarsToWrite, colnames(local.score)[seq_along(extraVarsToWrite)])

    # Test code for in-database computations
    system.time(
    {
        rxSetComputeContext(getSqlServerComputeContext())
        sqlserver.data <- getClaimsSqlServerDS()
        sqlserver.model <- rxOneClassSvm(form, data = sqlserver.data,
            transforms = transforms, rowSelection = rowSelection)
    })
    sqlserver.summ <- summary(sqlserver.model)
    sqlserver.score <- rxPredict(sqlserver.model, data = sqlserver.data,
        writeModelVars = writeModelVars, extraVarsToWrite = extraVarsToWrite)
    checkEquals(extraVarsToWrite, colnames(sqlserver.score)[seq_along(extraVarsToWrite)])

    # compare local and sqlserver
    #TODO (lgong) 2016-03-30: why outModelBin do not match?
    domatch <- c("Formula", "mamlCode", "Description", "summaryType")
    #dontmatch <- c("dataSource", "params", "call", "outModel", "outModelBin")
    checkEquals(local.model[domatch], sqlserver.model[domatch])

    checkEquals(coef(local.summ), coef(sqlserver.summ))
    checkEquals(local.score, sqlserver.score)
}

#############################################################################################
# Tests for running R scripts inside T-SQL (add SQL tests in SqlServerIOQ_Tests.sql first)
#############################################################################################
"rxuTestAnalysisFunctionWithTSQL" <- function(funName, captureLogging = FALSE, doPredict = FALSE)
{
    ## get the connectionString
    sqlCompute <- getSqlServerComputeContext()
    connectionString <- sqlCompute@connectionString

    ## extra the components from the connectionString
    connectionSubstr <- strsplit(connectionString, "=|;")[[1]]
    connection <- setNames(
        connectionSubstr[seq.int(from = 2, to = length(connectionSubstr), by = 2)],
        connectionSubstr[seq.int(from = 1, to = length(connectionSubstr), by = 2)])

    ## form the basic sqlcmd command line
    pathToSql <- gsub("/", "\\\\", system.file("IOQ", package = "MicrosoftML")) #sqlcmd cannot handle forward slash in path
    #pathToSql <- file.path(Sys.getenv("USERPROFILE"), "/Work/git/MicrosoftRML/MicrosoftML/inst/IOQ") 

    if ("Trusted_Connection" %in% names(connection))
    {
        sqlCmd <- sprintf(fmt = "sqlcmd -E -S \"%s\" -d \"%s\"", 
            connection["Server"], connection["Database"])
    } else {
        sqlCmd <- sprintf(fmt = "sqlcmd -U \"%s\" -P \"%s\" -S \"%s\" -d \"%s\"", 
        connection["Uid"], connection["Pwd"], connection["Server"], connection["Database"])
    }
    
    ## execute sqlcmd with specific sql script and scripting variables
    switch (funName,
        .setup = 
        {
            sqlScript <- file.path(pathToSql, "SqlServerIOQ_Tests_Setup.sql")    
            command <- sprintf(fmt = "%s -i \"%s\"", sqlCmd, sqlScript)
            logging <- system(command = command, intern = captureLogging)
            
            checkTrue(TRUE)
        },
        .teardown = 
        {
            sqlScript <- file.path(pathToSql, "SqlServerIOQ_Tests_Teardown.sql")
            command <- sprintf(fmt = "%s -i \"%s\"", sqlCmd, sqlScript)
            logging <- system(command = command, intern = captureLogging)
            
            checkTrue(TRUE)
        },
        {
            sqlScript <- file.path(pathToSql, "SqlServerIOQ_Tests.sql")
            command <- sprintf(fmt = "%s -i \"%s\" -v funname = %s connstr = \"%s\"", 
                sqlCmd, sqlScript, funName, connectionString)
            logging <- system(command = command, intern = captureLogging)
            
            sqlQuery  <- sprintf("SELECT * FROM SqlServerIOQ_Tests WHERE funname = '%s'", funName)
            sqlTable <- RxSqlServerData(sqlQuery = sqlQuery, connectionString = connectionString)
            sqlResult <- rxImport(inData = sqlTable)
            pass <- ifelse(nrow(sqlResult) > 0, sqlResult[["pass"]], FALSE)
            checkTrue(pass)
            
            if (doPredict) {
                sqlQuery  <- sprintf("SELECT * FROM SqlServerIOQ_Tests WHERE funname = '%s'", paste(funName, "rxPredict", sep = "_"))
                sqlTable <- RxSqlServerData(sqlQuery = sqlQuery, connectionString = connectionString)
                sqlResult <- rxImport(inData = sqlTable)
                pass <- ifelse(nrow(sqlResult) > 0, sqlResult[["pass"]], FALSE)
                checkTrue(pass)
            }
        }
    )
}

"test.SqlServer.TSQL.AAA.setup" <- function()
{ 
    sqlcmdInstalled <- try(system("sqlcmd -?", intern = TRUE))
    if (inherits(sqlcmdInstalled, "try-error")) stop(sqlcmdInstalled)
    
    rxuTestAnalysisFunctionWithTSQL(funName = ".setup", captureLogging = FALSE)
}

"test.SqlServer.TSQL.ZZZ.teardown" <- function()
{ 
    rxuTestAnalysisFunctionWithTSQL(funName = ".teardown", captureLogging = FALSE)
}


"test.SqlServer.TSQL.HPA.rxFastForest_binary" <- function()
{ 
    rxuTestAnalysisFunctionWithTSQL(funName = "rxFastForest_binary", captureLogging = FALSE, doPredict = TRUE)
}

"test.SqlServer.TSQL.HPA.rxFastForest_regression" <- function()
{ 
    rxuTestAnalysisFunctionWithTSQL(funName = "rxFastForest_regression", captureLogging = FALSE, doPredict = TRUE)
}

"test.SqlServer.TSQL.HPA.rxFastTrees_binary" <- function()
{ 
    rxuTestAnalysisFunctionWithTSQL(funName = "rxFastTrees_binary", captureLogging = FALSE, doPredict = TRUE)
}

"test.SqlServer.TSQL.HPA.rxFastTrees_regression" <- function()
{ 
    rxuTestAnalysisFunctionWithTSQL(funName = "rxFastTrees_regression", captureLogging = FALSE, doPredict = TRUE)
}

"test.SqlServer.TSQL.HPA.rxFastLinear_regression" <- function()
{ 
    rxuTestAnalysisFunctionWithTSQL(funName = "rxFastLinear_regression", captureLogging = FALSE, doPredict = TRUE)
}

"test.SqlServer.TSQL.HPA.rxFastLinear_binary" <- function()
{ 
    rxuTestAnalysisFunctionWithTSQL(funName = "rxFastLinear_binary", captureLogging = FALSE, doPredict = TRUE)
}

"test.SqlServer.TSQL.HPA.rxLogisticRegression_binary" <- function()
{ 
    rxuTestAnalysisFunctionWithTSQL(funName = "rxLogisticRegression_binary", captureLogging = FALSE, doPredict = TRUE)
}

"test.SqlServer.TSQL.HPA.rxLogisticRegression_multiClass" <- function()
{ 
    rxuTestAnalysisFunctionWithTSQL(funName = "rxLogisticRegression_multiClass", captureLogging = FALSE, doPredict = TRUE)
}

"test.SqlServer.TSQL.HPA.rxNeuralNet_binary" <- function()
{ 
    rxuTestAnalysisFunctionWithTSQL(funName = "rxNeuralNet_binary", captureLogging = FALSE, doPredict = TRUE)
}

"test.SqlServer.TSQL.HPA.rxNeuralNet_multiClass" <- function()
{ 
    rxuTestAnalysisFunctionWithTSQL(funName = "rxNeuralNet_multiClass", captureLogging = FALSE, doPredict = TRUE)
}

"test.SqlServer.TSQL.HPA.rxNeuralNet_regression" <- function()
{ 
    #DEACTIVATED("Defect 7281471: need to expose the maxNorm option for rxNeuralNet and document how to use it.")
    rxuTestAnalysisFunctionWithTSQL(funName = "rxNeuralNet_regression", captureLogging = FALSE, doPredict = TRUE)
}

"test.SqlServer.TSQL.HPA.rxOneClassSvm" <- function()
{ 
    #DEACTIVATED("Defect 7274035: ml Functions with transforms or rowSelection seem to hang if running locally in TSQL.")
    rxuTestAnalysisFunctionWithTSQL(funName = "rxOneClassSvm", captureLogging = FALSE, doPredict = TRUE)
}
