# This 'adult' data used here was extracted from the census bureau database found at
# http://www.census.gov/ftp/pub/DES/www/welcome.html
# More information can be found at:
# http://archive.ics.uci.edu/ml/machine-learning-databases/adult/adult.names
#
# If 'download' is set to TRUE and the data is not alread present locally, 
# it will be downloaded from:
# http://archive.ics.uci.edu/ml/machine-learning-databases/adult/
# and stored in the sampleDataDir

getAdultData <- function(download = TRUE, sampleDataDir = NULL, createDir = TRUE)
{
    sampleDataDir <- getSampleDataDir(sampleDataDir = sampleDataDir,
        createDir = createDir) 
    
    # Check to see if xdf files exist
    adultPath <- file.path(sampleDataDir, "adult")
    if (!file.exists(adultPath))
    {
        dir.create(adultPath)
    }
    
    trainPath <- file.path(adultPath, "adultTrain.xdf")
    testPath <- file.path(adultPath, "adultTest.xdf")
    if (file.exists(trainPath) && file.exists(testPath))
    {
        adult <- list(adult_train = rxImport(trainPath), adult_test = rxImport(testPath))
        return( adult )
    }
    if (!download)
    {
        stop(paste("Data files are not available in: ", adultPath))
    }
    else
    {
        cat("Downloading and importing data to:", adultPath, "\n")
    }
           
    adult.names.desc <- "http://archive.ics.uci.edu/ml/machine-learning-databases/adult/adult.names"
    adult.names <- try(read.table(url(adult.names.desc), sep = "*", comment.char = "|", stringsAsFactors = FALSE))
    stopifnot(!inherits(adult.names, "try-error"))

    label.levels <- rev(strsplit(adult.names[1, ], split = "[,|.]\\s*")[[1]]) #arrange the order
    label.name <- "income"
    adult.names.split <- strsplit(adult.names[-1, ], ":\\s*")
    col.names <- c(sapply(adult.names.split, "[", 1), label.name)
    col.names <- gsub( "-", ".", col.names)
    col.levels <- c(strsplit(sapply(adult.names.split, "[", 2), "[,|.]\\s*"), list(label.levels))
    names(col.levels) <- col.names
    col.factors <- sapply(col.levels, "length") != 1
    
    # Do some factor level order clean-up
    col.levels[["workclass"]] <- c("Private", "Federal-gov", "State-gov", "Local-gov", 
             "Self-emp-inc", "Self-emp-not-inc", "Without-pay", "Never-worked")
    col.levels[["education"]] <-  c("Doctorate", "Masters", "Bachelors", "Some-college",  
         "Prof-school", "Assoc-acdm", "Assoc-voc", "HS-grad", "12th", "11th", "10th", 
         "9th", "7th-8th",  "5th-6th", "1st-4th", "Preschool")     
    col.levels[["relationship"]] <- c("Wife", "Husband", "Own-child", "Other-relative", 
        "Unmarried", "Not-in-family")
    col.levels[["race"]] <- c("White", "Black", "Asian-Pac-Islander", "Amer-Indian-Eskimo", "Other") 
    
	col.levels[["native-country"]] = c("United-States", "Outlying-US(Guam-USVI-etc)", "Puerto-Rico", 
            "Canada", "Mexico", "Cuba",  "Jamaica", "Haiti", "Honduras","Guatemala",
            "Dominican-Republic", "Nicaragua", "El-Salvador", "Trinadad&Tobago", 
            "Peru", "Columbia", "Ecuador", 
            "England", "Ireland", "Scotland", "Holand-Netherlands", "Germany", "France", 
			"Portugal", "Greece","Italy", "Yugoslavia", "Hungary",  "Poland",  
			"Iran", "India", "South", "China",   "Cambodia", "Thailand", 
			"Vietnam", "Laos",  "Taiwan",  "Hong", "Japan", "Philippines")    
    adult.train.desc <- "http://archive.ics.uci.edu/ml/machine-learning-databases/adult/adult.data"
    adult.train <- read.csv(url(adult.train.desc), col.names = col.names, header = FALSE, 
        check.names = FALSE, strip.white = TRUE, comment.char = "|", stringsAsFactors = FALSE)
    for (i in seq_along(col.names)) {
        if (col.factors[i]) adult.train[[i]] <- factor(adult.train[[i]], levels = col.levels[[i]])
    }

    adult.test.desc <- "http://archive.ics.uci.edu/ml/machine-learning-databases/adult/adult.test"
    adult.test <- read.csv(url(adult.test.desc), col.names = col.names, header = FALSE, 
        check.names = FALSE, strip.white = TRUE, comment.char = "|", stringsAsFactors = FALSE)
    adult.test[[label.name]] <- gsub("[.]", "", adult.test[[label.name]])   #remove the extra "."
    for (i in seq_along(col.names)) {
        if (col.factors[i]) adult.test[[i]] <- factor(adult.test[[i]], levels = col.levels[[i]])
    }
    adult.train$IsOver50K <- adult.train$income == ">50K"
    adult.test$IsOver50K <- adult.test$income == ">50K" 
    
    # Save to local xdf files
    rxDataStep(inData = adult.train, outFile = trainPath)
    rxDataStep(inData = adult.test, outFile = testPath) 
    adult <- list(adult_train = adult.train, adult_test = adult.test)
    return( adult )
}

