# Data Set: German Credit Data
#
# For more information, see:
# http://archive.ics.uci.edu/ml/machine-learning-databases/statlog/german/german.doc
#
# If 'download' is set to TRUE and data is not available locally, 
# it will be downloaded from:
# http://archive.ics.uci.edu/ml/machine-learning-databases/statlog/german/
# and stored in the sampleDataDir

"getGermanCreditData" <- function(download = TRUE, sampleDataDir = NULL, createDir = TRUE)
{
    sampleDataDir <- getSampleDataDir(sampleDataDir = sampleDataDir,
        createDir = createDir) 
    
    # Check to see if xdf files exist
    gcPath <- file.path(sampleDataDir, "germanCredit")
    if (!file.exists(gcPath))
    {
        dir.create(gcPath)
    }
    
    trainPath <- file.path(gcPath, "germanCreditTrain.xdf")
    testPath <- file.path(gcPath, "germanCreditTest.xdf")
    if (file.exists(trainPath) && file.exists(testPath))
    {
        germanCredit <- list(
            germanCreditTrain = rxDataStep(trainPath), 
            germanCreditTest = rxDataStep(testPath))
        return( germanCredit )
    }
    if (!download)
    {
        stop(paste("Data files are not available in: ", gcPath))
    }
    else
    {
        cat("Downloading and importing data to:", gcPath, "\n")
    }
   
    germanColInfo <- list(
        V1 = list(newName = "AccountStatus", type = "factor", 
	              description = "Status of existing checking account in DM",
                  levels = paste0("A1", 1:4),
                  newLevels = c("<0", "[0-200)", ">=200 or salary", "No account")),
        V2 = list(newName = "Duration", type = "integer", 
	              description = "Duration in months"),
        V3 = list(newName = "CreditHistory", type = "factor",
	              description = "Credit history",
                  levels = paste0("A3", 0:4),
                  newLevels = c("Duly", "Duly This Bank", "Duly Til Now", "Delay", "Critical")),
        V4 = list(newName = "Purpose", type = "factor",
	            levels = paste0("A4", 0:10),
                newLevels = c("car (new)", "car (used)", "furniture/equipment",
	               "radio/TV", "appliances", "repairs", "education", "vacation",
                   "retraining", "business", "others")),
        V5 = list(newName = "CreditAmount", type = "integer"),
        V6 = list(newName = "Savings", type = "factor",
	            description = "Savings account/bonds in DM",
                levels = paste0("A6", 1:5),
                newLevels = c("<100", "[100-500)", "[500-1000)", ">=1000", "Unknown")),
        V7 = list(newName = "EmploymentYears", type = "factor",
	              description = "Present employment (in years) since",
                  levels = paste0("A7", 1:5),
                  newLevels = c("unemployed", "<1", "[1-4)", "[4-7)", ">=7")),
        V8 = list(newName = "InstallmentRate", type = "integer",
	              description = "Installment rate in percentage of disposable income"),
        V9 = list(newName = "StatusAndSex", type = "factor",
	              description = "Personal status and sex",
                  levels = paste0("A9", 1:5),
                  newLevels = c("M/divorced", "F/not single", "M/single",
                    "M/married", "F/single")),
        V10 = list(newName = "OtherDebtors", type = "factor",
                  description = "Other debtors/guarantors",
                  levels = paste0("A10", 1:3),
                  newLevels = c("none", "co-applicant", "guarantor")),
        V11 = list(newName = "YearsPresentResidence", type = "integer",
	              description = "Present residence since"),
        V12 = list(newName = "Property", type = "factor",
	               levels = paste0("A12", 1:4),
                   newLevels = c("real estate", "life insurance etc", "car", "unknown")),
        V13 = list(newName = "Age", type = "integer",
	               description = "Age in years"),
        V14 = list(newName = "OtherInstallment", type = "factor",
	               description = "Other installment plans",
                   levels = paste0("A14", 1:3),
                   newLevels = c("bank", "stores", "none")),
        V15 = list(newName = "Housing", type = "factor",
                   levels = paste0("A15", 1:3),
                   newLevels = c("rent", "own", "free")),
        V16 = list(newName = "ExistingCredits", type = "integer",
                   description = "Number of existing credits at this bank"),
        V17 = list(newName = "Job", type = "factor",
                   levels = paste0("A17", 1:4),
                   newLevels = c("unemployed/non-res", "unskilled resident", 
                       "skilled", "highly qualified")),
        V18 = list(newName = "LiablePeople", type = "integer",
	               description = "Number of people being liable to provide maintenance for"),
        V19 = list(newName = "Telephone", type = "factor",
                   levels = paste0("A19", 1:2),
                   newLevels = c("none", "yes")),
        V20 = list(newName = "ForeignWorker", type = "factor",
	               levels = paste0("A20", 1:2),
                   newLevels = c("yes", "no")),
        V21 = list(newName = "Bad", type = "integer") #Note: 1 = good, 2= bad
        )

    germanCreditDataUrl <- "http://archive.ics.uci.edu/ml/machine-learning-databases/statlog/german/german.data"    
    germanCreditData <- rxImport(germanCreditDataUrl, 
      firstRowIsColNames = FALSE,
      colInfo = germanColInfo)
    stopifnot(!inherits(germanCreditData, "try-error"))
    # Convert Bad to logical
    germanCreditData$Bad <- germanCreditData$Bad == 2  
   
    splitGermanData <- rxSplit(inData = germanCreditData, 
       splitByFactor = "splitVar", 
       transforms=list(splitVar = factor( 
        sample(0:1,  size=.rxNumRows, replace=TRUE, prob=c(.75, .25)), 
        levels=0:1, labels = c("Train", "Test"))), 
      overwrite = TRUE)
	
    # Remove the temporary 'splitVar', the last column
    numCols <- ncol(splitGermanData[[1]])  
    
    # Save to local xdf files  
    rxDataStep(inData = splitGermanData[[1]][, -numCols], outFile = trainPath) 
    rxDataStep(inData = splitGermanData[[2]][, -numCols], outFile = testPath) 
          
    germanCredit <- list(train =  splitGermanData[[1]][, -numCols], 
        test = splitGermanData[[2]][, -numCols])

    return( germanCredit )
}