# Copyright (c) Microsoft Corporation.  All rights reserved.
# Tests for featurizeText transform

#function(vars, language = "English", stopwordsRemover = NULL, case = "lower",
#              keepDiacritics = FALSE, keepPunctuations = TRUE, keepNumbers = TRUE,
#              ngramLength = 1, skipLength = 0, dictionary = NULL,
#              wordFeatureExtractor = ngramCount(), charFeatureExtractor = NULL, vectorNormalizer = "l2",
#    ...)

"test.featurizeText.args.removeStopWords.dictionary" <- function() {
    myData <- data.frame(opinion = c(
        "I love it",
        "I love it",
        "Love it",
        "I love it a lot",
        "Really love it",
        "I love it",
        "I love it",
        "love it",
        "I love it a lot",
        "Really love it",
        "I hate it",
        "I hate it",
        "I hate it",
        "hate it",
        "hate and",
        "I hate it and",
        "I hate it and",
        "I hate it and",
        "hate it and",
        "hate and"),
        like = rep(c(TRUE, FALSE), each = 10),
        stringsAsFactors = FALSE)

    outModel1 <- rxLogisticRegression(like ~ opinionCount, data = myData,
        mlTransforms = list(featurizeText(vars = c(opinionCount = "opinion"))))
    checkEquals(names(coef(outModel1)), c("(Bias)", "hate", "love", "and"))

    outModel2 <- rxLogisticRegression(like ~ opinionCount, data = myData,
        mlTransforms = list(featurizeText(vars = c(opinionCount = "opinion"),
            stopwordsRemover = stopwordsDefault())))
    checkEquals(names(coef(outModel2)), c("(Bias)", "hate", "love"))

    # Illegal value
    outModel3 <- try(
        rxLogisticRegression(like ~ opinionCount, data = myData,
        mlTransforms = list(featurizeText(vars = c(opinionCount = "opinion"),
            stopwordsRemover = 1))), silent = TRUE)
    checkTrue(inherits(outModel3, "try-error"))
    checkTrue(grepl("Invalid stopwordsRemover type", as.character(outModel3)))

    stopwords <- c("and", "!")
    lapply(stopwords, write, "stopwords.txt", append = TRUE)
    outModel4 <- rxLogisticRegression(like ~ opinionCount, data = myData,
        mlTransforms = list(featurizeText(vars = c(opinionCount = "opinion"),
            stopwordsRemover = stopwordsCustom(dataFile = "stopwords.txt"))))
    checkEquals(names(coef(outModel4)), c("(Bias)", "hate", "love"))

    outModel5 <- rxLogisticRegression(like ~ opinionCount, data = myData,
        mlTransforms = list(featurizeText(vars = c(opinionCount = "opinion"),
            dictionary = termDictionary(terms = "and,!"))))
    checkEquals(names(coef(outModel5)), c("(Bias)", "and"))
}

"test.featurizeText.args.language" <- function() {
    myData <- data.frame(opinion = c(
        "I love it ta the",
        "I love it ta ta the",
        "Love ta it the",
        "I love it a lot ta the",
        "ta Really love it! the",
        "I hate it ni",
        "I hate it ni ni",
        "I hate it ni",
        "Hate it",
        "ni Hate"),
        like = rep(c(TRUE, FALSE), each = 5),
        stringsAsFactors = FALSE)

    outModel1 <- rxLogisticRegression(like ~ opinionCount, data = myData,
        mlTransforms = list(featurizeText(vars = c(opinionCount = "opinion"), language = "English",
            stopwordsRemover = stopwordsDefault())))
    # 'the' should be removed for English
    checkEquals(sort(names(coef(outModel1))), sort(c("(Bias)", "hate", "love", "ni", "ta")))

    outModel2 <- rxLogisticRegression(like ~ opinionCount, data = myData,
        mlTransforms = list(featurizeText(vars = c(opinionCount = "opinion"), language = "French",
            stopwordsRemover = stopwordsDefault())))
    # 'ni' and 'ta' should be removed for French
    checkEquals(sort(names(coef(outModel2))), sort(c("(Bias)", "hate", "love", "the")))

    outModel3 <- try(
        rxLogisticRegression(like ~ opinionCount, data = myData,
            mlTransforms = list(featurizeText(vars = c(opinionCount = "opinion"), language = "Swahili",
            stopwordsRemover = stopwordsDefault()))), silent = TRUE)

    checkTrue(inherits(outModel3, "try-error"))
}

"test.featurizeText.args.tokenizer" <- function() {
    examples <- c(
        "ababababababababab",
        "abcbabcbabcbabcbabc",
        "aaaabbbbccccaaabbbb",
        "abababababab",
        "abcabc",
        "aabbaabbaabbaabb",
        "abdbabdbabdbabdb",
        "aaabbbddddddbbbaaaa",
        "aabbaabb",
        "abd")
    myData <- data.frame(opinion = examples,
        like = rep(c(TRUE, FALSE), each = 5),
        stringsAsFactors = FALSE)

    outModel1 <- rxLogisticRegression(like ~ opinionCount, data = myData, l1Weight = 0.4, optTol = 0.0001, memorySize = 5,
        mlTransforms = list(featurizeText(vars = c(opinionCount = "opinion"), charFeatureExtractor = ngramCount(), wordFeatureExtractor = NULL)))
    # The model contains the coefficients corresponding to 'c' and 'd'.
    checkEquals(sort(names(coef(outModel1))), sort(c("(Bias)", "c", "d")))

    outModel2 <- rxLogisticRegression(like ~ opinionCount, data = myData, l1Weight = 0.4, optTol = 0.0001, memorySize = 5,
        mlTransforms = list(featurizeText(vars = c(opinionCount = "opinion"), wordFeatureExtractor = ngramCount())))
    # The model contains the coefficients corresponding to all the words in the data.
    checkEquals(sort(names(coef(outModel2))), sort(append("(Bias)", examples)))

    outModel3 <- rxLogisticRegression(like ~ opinionCount, data = myData, l1Weight = 0.3, optTol = 0.0001, memorySize = 5,
        mlTransforms = list(featurizeText(vars = c(opinionCount = "opinion"), charFeatureExtractor = ngramCount(ngramLength = 3), wordFeatureExtractor = NULL)))
    # The model contains the coefficients corresponding to 'c' and 'd', 'b|c' and 'b|d' and 'a|b|a', 'b|a|b' and 'a|b|c'.
    #NOTE: The sort order for character vectors will depend on the collating sequence of the locale in use.
    #print(Sys.getlocale())
    #print(sort(names(coef(outModel3))))
    #checkEquals(sort(names(coef(outModel3))), c("a|b|a", "a|b|c", "b|a|b", "b|c", "b|d", "Bias", "c", "d"))
    checkTrue(setequal(names(coef(outModel3)), c("a|b|a", "a|b|c", "b|a|b", "b|c", "b|d", "(Bias)", "c", "d")))

    outModel4 <- try(
        rxLogisticRegression(like ~ opinionCount, data = myData,
            mlTransforms = list(featurizeText(vars = c(opinionCount = "opinion"), wordFeatureExtractor = NULL, charFeatureExtractor = NULL))), silent = TRUE)
    checkTrue(inherits(outModel4, "try-error"))
}

"test.featurizeText.args.case" <- function() {
    myData <- data.frame(opinion = c(
        "I LOVE it",
        "I LOVE it",
        "LOVE it",
        "I LOVE it a lot",
        "Really LOVE it",
        "I love it",
        "I love it",
        "love it",
        "I love it a lot",
        "Really love it",
        "I HATE it",
        "I HATE it",
        "I HATE it",
        "HATE it",
        "hate",
        "I hate it",
        "I hate it",
        "I hate it",
        "hate it",
        "hate"),
        like = rep(c(TRUE, FALSE), each = 10),
        stringsAsFactors = FALSE)

    outModel1 <- rxLogisticRegression(like ~ opinionCount, data = myData,
        mlTransforms = list(featurizeText(vars = c(opinionCount = "opinion"), case = "none")))

    checkEquals(names(coef(outModel1)), c("(Bias)", "hate", "LOVE", "love", "HATE"))

    outModel2 <- rxLogisticRegression(like ~ opinionCount, data = myData,
        mlTransforms = list(featurizeText(vars = c(opinionCount = "opinion"), case = "upper")))

    checkEquals(names(coef(outModel2)), c("(Bias)", "HATE", "LOVE"))

    outModel3 <- rxLogisticRegression(like ~ opinionCount, data = myData,
        mlTransforms = list(featurizeText(vars = c(opinionCount = "opinion"), case = "lower")))

    checkEquals(names(coef(outModel3)), c("(Bias)", "hate", "love"))

    outModel4 <- try(
        rxLogisticRegression(like ~ opinionCount, data = myData,
            mlTransforms = list(featurizeText(vars = c(opinionCount = "opinion"), case = "lowercase"))),
        silent = TRUE)
    checkTrue(inherits(outModel4, "try-error"))
}

"test.featurizeText.args.keepDiacritics" <- function() {
    DEACTIVATED("Need test for keepDiacritics")
}

"test.featurizeText.args.keepPunctuation" <- function() {
    myData <- data.frame(opinion = c(
        "I love it!",
        "I love it!",
        "Love it!",
        "I love it a lot!",
        "Really love it!",
        "I hate it",
        "I hate it",
        "I hate it.",
        "Hate it",
        "Hate"),
        like = rep(c(TRUE, FALSE), each = 5),
        stringsAsFactors = FALSE)

    outModel1 <- rxLogisticRegression(like ~ opinionCount, data = myData,
        mlTransforms = list(featurizeText(vars = c(opinionCount = "opinion"), keepPunctuation = TRUE)))
    # 'hate', 'love', and '!' have non-zero weights
    checkEquals(names(coef(outModel1)), c("(Bias)", "hate", "love", "!"))

    outModel2 <- rxLogisticRegression(like ~ opinionCount, data = myData,
        mlTransforms = list(featurizeText(vars = c(opinionCount = "opinion"), keepPunctuation = FALSE)))
    checkEquals(names(coef(outModel2)), c("(Bias)", "hate", "love"))

    # Invalid value
    outModel3 <- try(
        rxLogisticRegression(like ~ opinionCount, data = myData,
           mlTransforms = list(featurizeText(vars = c(opinionCount = "opinion"),
           keepPunctuation = "yes"))), silent = TRUE)
    checkTrue(inherits(outModel3, "try-error"))
    checkTrue(grepl("must be a logical", as.character(outModel3)))
}

"test.featurizeText.args.keepNumbers" <- function() {
    myData <- data.frame(opinion = c(
        "I love it 1",
        "I love it 1",
        "Love it 1",
        "I love it a lot 1",
        "Really love it 1",
        "I hate it 2",
        "I hate it 2",
        "I hate it 2",
        "Hate it",
        "Hate"),
        like = rep(c(TRUE, FALSE), each = 5),
        stringsAsFactors = FALSE)

    outModel1 <- rxLogisticRegression(like ~ opinionCount, data = myData,
        mlTransforms = list(featurizeText(vars = c(opinionCount = "opinion"), keepNumbers = TRUE)))
    # 'hate', 'love', and '!' have non-zero weights
    checkEquals(names(coef(outModel1)), c("(Bias)", "hate", "love", "1"))

    outModel2 <- rxLogisticRegression(like ~ opinionCount, data = myData,
        mlTransforms = list(featurizeText(vars = c(opinionCount = "opinion"), keepNumbers = FALSE)))
    checkEquals(names(coef(outModel2)), c("(Bias)", "hate", "love"))

    # Invalid value
    outModel3 <- try(
        rxLogisticRegression(like ~ opinionCount, data = myData,
           mlTransforms = list(featurizeText(vars = c(opinionCount = "opinion"),
           keepNumbers = NULL))), silent = TRUE)
    checkTrue(inherits(outModel3, "try-error"))
    checkTrue(grepl("must be a logical", as.character(outModel3)))
}

"test.featurizeText.args.ngramLength" <- function() {
    myData <- data.frame(opinion = c(
        "I do like it",
        "I do like it",
        "Like it",
        "I do like it a lot",
        "Really like it",
        "do like it.",
        "like",
        "do like",
        "I really do like it",
        "do like it",
        "I do not like it",
        "don't like it",
        "I don't like it",
        "Do not like it",
        "Don't like",
        "I do not like it.",
        "do not like",
        "do not like at all",
        "I really do not like it",
        "do not like it"),
        like = rep(c(TRUE, FALSE), each = 10),
        stringsAsFactors = FALSE)

    outModel1 <- rxLogisticRegression(like ~ opinionCount, data = myData,
        mlTransforms = list(featurizeText(vars = c(opinionCount = "opinion"), wordFeatureExtractor = ngramCount(ngramLength = 1))))
    checkEquals(names(coef(outModel1)), c("(Bias)", "not"))

    outModel2 <- rxLogisticRegression(like ~ opinionCount, data = myData,
        mlTransforms = list(featurizeText(vars = c(opinionCount = "opinion"), wordFeatureExtractor = ngramCount(ngramLength = 2))))
    checkEquals(names(coef(outModel2)), c("(Bias)", "do|like", "do|not", "not", "not|like"))

    # Try something unreasonable
    outModel3 <- try(
        rxLogisticRegression(like ~ opinionCount, data = myData,
        mlTransforms = list(featurizeText(vars = c(opinionCount = "opinion"), wordFeatureExtractor = ngramCount(ngramLength = 200)))),
        silent = TRUE)
    checkTrue(inherits(outModel3, "try-error"))
    checkTrue(grepl("less than or equal to 10", as.character(outModel3)))

    outModel4 <- try(
        rxLogisticRegression(like ~ opinionCount, data = myData,
        mlTransforms = list(featurizeText(vars = c(opinionCount = "opinion"), wordFeatureExtractor = ngramCount(ngramLength = 0)))),
        silent = TRUE)
    checkTrue(inherits(outModel4, "try-error"))
    checkTrue(grepl("greater than or equal to 1", as.character(outModel4)))
}

"test.featurizeText.args.skipLength" <- function() {
    myData <- data.frame(opinion = c(
        "I do like it",
        "I do like it",
        "Like it",
        "I do like it a lot",
        "Really like it",
        "do like it.",
        "like",
        "do like",
        "I really do like it",
        "do like it",
        "I do not like it",
        "don't like it",
        "I don't like it",
        "Do not like it",
        "Don't like",
        "I do not like it.",
        "do not like",
        "do not like at all",
        "I really do not like it",
        "do not like it"),
        like = rep(c(TRUE, FALSE), each = 10),
        stringsAsFactors = FALSE)

    outModel1 <- rxLogisticRegression(like ~ opinionCount, data = myData,
        mlTransforms = list(featurizeText(vars = c(opinionCount = "opinion"), wordFeatureExtractor =
        ngramCount(ngramLength = 2, skipLength = 0))))
    checkEquals(names(coef(outModel1)), c("(Bias)", "do|like", "do|not", "not", "not|like"))

    outModel2 <- rxLogisticRegression(like ~ opinionCount, data = myData,
        mlTransforms = list(featurizeText(vars = c(opinionCount = "opinion"), wordFeatureExtractor =
        ngramCount(ngramLength = 2, skipLength = 1))))
    checkEquals(names(coef(outModel2)), c("(Bias)", "do|not", "not", "not|like"))

    # Try something unreasonable
    outModel3 <- try(
        rxLogisticRegression(like ~ opinionCount, data = myData,
        mlTransforms = list(featurizeText(vars = c(opinionCount = "opinion"), wordFeatureExtractor =
        ngramCount(ngramLength = 2, skipLength = 10)))), silent = TRUE)
    checkTrue(inherits(outModel3, "try-error"))
    checkTrue(grepl("less than or equal to 10", as.character(outModel3)))
}

#ngramCount(maxNumTerms = 10000000, weighting = "tf")
"test.featurizeText.args.wordFeatureExtractor.ngramCount" <- function() {
    myData <- data.frame(opinion = c(
        "I love it!",
        "I love it!",
        "Love it!",
        "I love it a lot!",
        "Really love it!",
        "I hate it",
        "I hate it",
        "I hate it.",
        "Hate it",
        "Hate"),
        like = rep(c(TRUE, FALSE), each = 5),
        stringsAsFactors = FALSE)

    outModel1 <- rxLogisticRegression(like ~ opinionCount, data = myData,
        mlTransforms = list(featurizeText(vars = c(opinionCount = "opinion"),
            wordFeatureExtractor = ngramCount())))
    checkEquals(names(coef(outModel1)), c("(Bias)", "hate", "love", "!"))

    outModel2 <- rxLogisticRegression(like ~ opinionCount, data = myData,
        mlTransforms = list(featurizeText(vars = c(opinionCount = "opinion"),
            wordFeatureExtractor = ngramCount(maxNumTerms = 5))))
    checkEquals(names(coef(outModel2)), c("(Bias)", "love", "!"))

    outModel3 <- rxLogisticRegression(like ~ opinionCount, data = myData,
        mlTransforms = list(featurizeText(vars = c(opinionCount = "opinion"),
            wordFeatureExtractor = ngramCount(maxNumTerms = 5, weighting = "tf"))))
    checkEquals(names(coef(outModel3)), c("(Bias)", "love", "!"))

    outModel4 <- rxLogisticRegression(like ~ opinionCount, data = myData,
        mlTransforms = list(featurizeText(vars = c(opinionCount = "opinion"),
            wordFeatureExtractor = ngramCount(weighting = "idf"))))
    checkEquals(names(coef(outModel4)), c("(Bias)", "hate", "love", "!"))

    outModel5 <- rxLogisticRegression(like ~ opinionCount, data = myData,
        mlTransforms = list(featurizeText(vars = c(opinionCount = "opinion"),
            wordFeatureExtractor = ngramCount(weighting = "tfidf"))))
    checkEquals(names(coef(outModel5)), c("(Bias)", "hate", "love", "!"))

    outModel5.old <- rxLogisticRegression(like ~ opinionCount, data = myData,
        mlTransforms = list(featurizeText(vars = c(opinionCount = "opinion"),
            ngramLength = 1, skipLength = 0,
            featureExtractor = ngramCount(weighting = "tfidf"))))
    checkEquals(names(coef(outModel5.old)), c("(Bias)", "hate", "love", "!"))

    domatch <- c("formula", "dataframeName", "mamlCode", "params", "Description", "summaryType", "coefficients")
    #dontmatch <- c("outModelBin", "call", "cachedSummary")
    checkEquals(outModel5[domatch], outModel5.old[domatch])

    checkException(outModel5.old <- rxLogisticRegression(like ~ opinionCount, data = myData,
        mlTransforms = list(featurizeText(vars = c(opinionCount = "opinion"),
            ngramLength = 1, skipLength = 0, wordFeatureExtractor = ngramCount(), 
            featureExtractor = ngramCount(weighting = "tfidf")))))

    # Try invalid arg value
    outModel6 <- try(
        rxLogisticRegression(like ~ opinionCount, data = myData,
        mlTransforms = list(featurizeText(vars = c(opinionCount = "opinion"),
            wordFeatureExtractor = ngramCount(weighting = "tfdf")))), silent = TRUE)
    checkTrue(inherits(outModel6, "try-error"))
    checkTrue(grepl("must be one of: 'tf", as.character(outModel6)))

    # Try invalid arg name
    outModel7 <- try(
        rxLogisticRegression(like ~ opinionCount, data = myData,
        mlTransforms = list(featurizeText(vars = c(opinionCount = "opinion"),
            wordFeatureExtractor = ngramCount(hashBits = 24)))),
        silent = TRUE)
    checkTrue(inherits(outModel7, "try-error"))
    checkTrue(grepl("unused argument", as.character(outModel7)))
}

#ngramHash(hashBits = 16, seed = 314489979, ordered = TRUE, invertHash = 0)  
"test.featurizeText.args.wordFeatureExtractor.ngramHash" <- function() {
    myData <- data.frame(opinion = c(
        "I love it!",
        "I love it!",
        "Love it!",
        "I love it a lot!",
        "Really love it!",
        "I hate it",
        "I hate it",
        "I hate it.",
        "Hate it",
        "Hate"),
        like = rep(c(TRUE, FALSE), each = 5),
        stringsAsFactors = FALSE)

    outModel1 <- rxLogisticRegression(like ~ opinionCount, data = myData,
        mlTransforms = list(featurizeText(vars = c(opinionCount = "opinion"),
            wordFeatureExtractor = ngramHash())))
    checkEquals(length(coef(outModel1)), 4)

    outModel2 <- rxLogisticRegression(like ~ opinionCount, data = myData,
        mlTransforms = list(featurizeText(vars = c(opinionCount = "opinion"),
            wordFeatureExtractor = ngramHash(invertHash = -1))))
    checkEquals(names(coef(outModel2)), c("(Bias)", "hate", "!", "love"))

    outModel3 <- rxLogisticRegression(like ~ opinionCount, data = myData,
        mlTransforms = list(featurizeText(vars = c(opinionCount = "opinion"),
            wordFeatureExtractor = ngramHash(invertHash = -1, hashBits = 3))))
    checkEquals(names(coef(outModel3)), c("(Bias)", "love", "!"))

    outModel4 <- rxLogisticRegression(like ~ opinionCount, data = myData,
        mlTransforms = list(featurizeText(vars = c(opinionCount = "opinion"),
            wordFeatureExtractor = ngramHash(invertHash = -1, seed = 7))))
    checkEquals(names(coef(outModel4)), c("(Bias)", "hate", "love", "!"))

    outModel5 <- rxLogisticRegression(like ~ opinionCount, data = myData,
        mlTransforms = list(featurizeText(vars = c(opinionCount = "opinion"),
            wordFeatureExtractor = ngramHash(invertHash = -1, ordered = TRUE))))
    checkEquals(names(coef(outModel5)), c("(Bias)", "hate", "!", "love"))

    # Mis-spell
    outModel6 <- try(
        rxLogisticRegression(like ~ opinionCount, data = myData,
            mlTransforms = list(featurizeText(vars = c(opinionCount = "opinion"),
                wordFeatureExtractor = maNgrmHash(invertHash = -1, ordered = TRUE)))),
            silent = TRUE)
    checkTrue(inherits(outModel6, "try-error"))
    checkTrue(grepl("could not find function", as.character(outModel6)))
}

"test.featurizeText.utf8" <- function() {
    if (getOption("encoding") != "UTF-8")
        DEACTIVATED("set the encoding to 'UTF-8', options(encoding='UTF-8'), to run this test.")

    myData <- data.frame(opinion = c(
        "I lóve it!",
        "I lóve it!",
        "Lóve it!",
        "I lóve it a lot!",
        "Really lóve it!",
        "I hate it",
        "I hate it",
        "I hate it.",
        "Hate it",
        "Hate"),
        like = rep(c(TRUE, FALSE), each = 5),
        stringsAsFactors = FALSE)

    outModel3 <- rxLogisticRegression(like ~ opinionCount, data = myData,
        mlTransforms = list(featurizeText(vars = c(opinionCount = "opinion"), keepDiacritics = TRUE,
            wordFeatureExtractor = ngramHash(invertHash = -1, hashBits = 3))))
    checkEquals(names(coef(outModel3)), c("(Bias)", "!", "lóve"))
}
