# Copyright (c) Microsoft Corporation.  All rights reserved.

# Tests for basic functionality of rxNeuralNet
".rxSqliteConnectionStringFromFile" <- function() {
    if (exists("rxSqliteConnectionStringFromFile", globalenv()))
        get("rxSqliteConnectionStringFromFile", globalenv())
    else
        RevoScaleR:::rxSqliteConnectionStringFromFile
    }

"test.rxNeuralNet.basic.binary" <- function() {
    # Create temporary data sources
    myIris = iris
    myIris$Setosa <- iris$Species == "setosa"

    myXdf <- tempfile(pattern = "myIris", fileext = ".xdf")
    on.exit(file.remove(myXdf), add = TRUE)
    rxDataStep(myIris, myXdf, rowsPerRead = 50, overwrite = TRUE)
    myXdfDS <- RxXdfData(file = myXdf)

    myCsv <- tempfile(pattern = "myIris", fileext = ".csv")
    on.exit(file.remove(myCsv), add = TRUE)
    myCsvDS <- RxTextData(file = myCsv, rowsPerRead = 40)
    rxDataStep(myIris, myCsvDS, overwrite = TRUE)

    readVars <- c("Setosa", "Sepal.Length", "Sepal.Width", "Petal.Width")
    # Run examples
    res1 <- rxNeuralNet(formula = Setosa ~ Sepal.Length + Sepal.Width + Petal.Width,
        data = myXdfDS)
    checkEquals(res1$params$readVars, readVars)
    scoreOut <- rxPredict(res1, data = myXdfDS, extraVarsToWrite = "Setosa")
    checkTrue(is.list(coef(res1)))

    res2 <- rxNeuralNet(formula = Setosa ~ Sepal.Length + Sepal.Width + Petal.Width,
        data = myIris)
    checkEquals(res2$params$readVars, readVars)

    res3 <- rxNeuralNet(formula = Setosa ~ Sepal.Length + Sepal.Width + Petal.Width,
        data = myCsvDS)
    checkEquals(res3$params$readVars, readVars)
}

"test.rxNeuralNet.basic.regression" <- function() {
    summary(fm1 <- lm(rating ~ ., data = attitude))

    # Create temporary data sources
    myXdf <- tempfile(pattern = "tempAttitude", fileext = ".xdf")
    on.exit(file.remove(myXdf), add = TRUE)
    rxDataStep(attitude, myXdf, rowsPerRead = 50, overwrite = TRUE)
    myXdfDS <- RxXdfData(file = myXdf)

    myCsv <- tempfile(pattern = "tempAttitude", fileext = ".csv")
    on.exit(file.remove(myCsv), add = TRUE)
    myCsvDS <- RxTextData(file = myCsv, rowsPerRead = 40)
    rxDataStep(attitude, myCsvDS, overwrite = TRUE)

    attitudeForm <- rating ~ complaints + privileges + learning + raises + critical + advance

    # Xdf example
    res1 <- rxNeuralNet(formula = attitudeForm, data = myXdfDS, type = "regression")
    scoreOut1 <- rxPredict(res1, data = myXdfDS, extraVarsToWrite = "rating")
    #rxLinePlot(rating~Score, type = c("p","r"), data = scoreOut1)
    checkTrue(is.list(coef(res1)))

    res2 <- rxNeuralNet(formula = attitudeForm, type = "regression",
        data = attitude)
    scoreOut2 <- rxPredict(res2, data = attitude, extraVarsToWrite = "rating")
    checkEquals(scoreOut1, scoreOut2)

    res3 <- rxNeuralNet(formula = attitudeForm, type = "regression",
        data = myCsvDS)
    scoreOut3 <- rxPredict(res2, data = myCsvDS, extraVarsToWrite = "rating")
    checkEquals(scoreOut1, scoreOut3, tolerance = .001)


}
"test.rxNeuralNet.basic.multi" <- function() {
    res1 <- rxNeuralNet(formula = Species ~ Sepal.Length + Sepal.Width + Petal.Length + Petal.Width,
        data = iris, type = "multi")
    scoreOut <- rxPredict(res1, data = iris, extraVarsToWrite = "Species")
    checkTrue(!is.null(scoreOut$PredictedLabel))

    checkTrue(is.list(coef(res1)))
}

"test.rxNeuralNet.args.optimizer.sgd" <- function() {
    #sgd(learningRate = 0.001, momentum = 0.0, nag = FALSE, weightDecay = 0.0, 
    #lRateRedRatio = 1.0, lRateRedFreq = 100, lRateRedErrorRatio = 0.0)

    myIris = iris
    myIris$isVersi <- iris$Species == "versicolor"

    lrates <- c(.002, .1, .15, .2, .4, 4)
    numRates <- length(lrates)
    diffs <- vector(mode = "numeric", length = numRates)

    for (i in seq_along(lrates)) {

        res1 <- rxNeuralNet(formula = isVersi ~ Sepal.Length + Sepal.Width + Petal.Width,
            data = myIris,
            optimizer = sgd(learningRate = lrates[i]))

        rxPredict1 <- rxPredict(res1, data = myIris, extraVarsToWrite = "isVersi")
        diffs[i] <- sum(rxPredict1$isVersi != rxPredict1$PredictedLabel)
    }

    # Extreme values should produce fewer accurate predictions

    checkTrue(diffs[1] > diffs[as.integer(numRates / 2)])
    checkTrue(diffs[numRates] > diffs[as.integer(numRates / 2)])

    # momentum
    # Valid ranges 0 <= x < 1
    momentumVals <- c(-1, 0, .1, .5, 1)
    isValid <- c(FALSE, TRUE, TRUE, TRUE, FALSE)
    numVals <- length(momentumVals)
    diffs <- vector(mode = "numeric", length = numVals)
    for (i in seq_along(momentumVals)) {
        out1 <- try(
        res1 <- rxNeuralNet(formula = isVersi ~ Sepal.Length + Sepal.Width + Petal.Width,
            data = myIris,
            optimizer = sgd(momentum = momentumVals[i])), silent = TRUE)
        if (!isValid[i]) {
            checkTrue(inherits(out1, "try-error"))
            checkTrue(grepl("'momentum' must be", as.character(out1)))
            diffs[i] <- as.numeric(NA)
        }
        else {
            rxPredict1 <- rxPredict(res1, data = myIris, extraVarsToWrite = "isVersi")
            diffs[i] <- sum(rxPredict1$isVersi != rxPredict1$PredictedLabel)
        }
    }

    #nag
    nagVals <- c(TRUE, FALSE, NA)
    numVals <- length(nagVals)
    isValid <- c(TRUE, TRUE, FALSE)
    diffs <- vector(mode = "numeric", length = numVals)
    for (i in seq_along(nagVals)) {

        out1 <- try(
        res1 <- rxNeuralNet(formula = isVersi ~ Sepal.Length + Sepal.Width + Petal.Width,
            data = myIris,
            optimizer = sgd(nag = nagVals[i])), silent = TRUE)
        if (!isValid[i]) {
            checkTrue(inherits(out1, "try-error"))
            checkTrue(grepl("'nag' must be", as.character(out1)))
            diffs[i] <- as.numeric(NA)
        }
        else {
            rxPredict1 <- rxPredict(res1, data = myIris, extraVarsToWrite = "isVersi")
            diffs[i] <- sum(rxPredict1$isVersi != rxPredict1$PredictedLabel)
        }
    }

    # weightDecay
    # Valid ranges 0 <= x < 1
    weightDecayVals <- c(-1, 0, .1, .5, 1)
    isValid <- c(FALSE, TRUE, TRUE, TRUE, FALSE)
    numVals <- length(weightDecayVals)
    diffs <- vector(mode = "numeric", length = numVals)
    for (i in seq_along(weightDecayVals)) {
        out1 <- try(
        res1 <- rxNeuralNet(formula = isVersi ~ Sepal.Length + Sepal.Width + Petal.Width,
            data = myIris,
            optimizer = sgd(weightDecay = weightDecayVals[i])), silent = TRUE)
        if (!isValid[i]) {
            checkTrue(inherits(out1, "try-error"))
            checkTrue(grepl("'weightDecay' must be", as.character(out1)))
            diffs[i] <- as.numeric(NA)
        }
        else {
            rxPredict1 <- rxPredict(res1, data = myIris, extraVarsToWrite = "isVersi")
            diffs[i] <- sum(rxPredict1$isVersi != rxPredict1$PredictedLabel)
        }
    }

    #lRateRedRatio = 1.0 : no Reduction, lRateRedFreq = 100, lRateRedErrorRatio = 0.0
    # FIXME: What are valid ranges for lRateRedFreq
    lRatioVals <- c(1.2, 1, .9, .8, .8, .8, .8, .7, .7, .7, 0)
    lFreqVals <- c(100, 100, 100, 10, 5, -1, 1, 1, 1, 1, 0)
    lErrorVals <- c(0, 0, 0, 0, 0, 0, .1, .8, 1, 1.0, 0)
    isValid <- c(FALSE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE)
    numVals <- length(lRatioVals)
    diffs <- vector(mode = "numeric", length = numVals)
    for (i in seq_along(lRatioVals)) {
        print(i)
        out1 <- try(
        res1 <- rxNeuralNet(formula = isVersi ~ Sepal.Length + Sepal.Width + Petal.Width,
            data = myIris,
            optimizer = sgd(lRateRedRatio = lRatioVals[i],
               lRateRedFreq = lFreqVals[i], lRateRedErrorRatio = lErrorVals[i]),
               reportProgress = 0, verbose = 0), silent = TRUE)
        if (!isValid[i]) {
            checkTrue(inherits(out1, "try-error"))
            diffs[i] <- as.numeric(NA)
        }
        else {
            rxPredict1 <- rxPredict(res1, data = myIris, extraVarsToWrite = "isVersi",
              reportProgress = 0, verbose = 0)
            diffs[i] <- sum(rxPredict1$isVersi != rxPredict1$PredictedLabel)
        }
    }

}

"test.rxNeuralNet.args.optimizer.ada" <- function() {

    #adaDeltaSgd(decay = .95, conditioningConst = 1E-06)    
    # decay
    # Valid ranges 0 < x < 1
    decayVals <- c(-1, 0, .1, .5, 1)
    isValid <- c(FALSE, FALSE, TRUE, TRUE, FALSE)
    numVals <- length(decayVals)
    diffs <- vector(mode = "numeric", length = numVals)
    for (i in seq_along(decayVals)) {
        out1 <- try(
        res1 <- rxNeuralNet(formula = Species ~ Sepal.Length + Sepal.Width + Petal.Width,
            data = iris, type = "multi",
            optimizer = adaDeltaSgd(decay = decayVals[i])), silent = TRUE)
        if (!isValid[i]) {
            checkTrue(inherits(out1, "try-error"))
            checkTrue(grepl("'decay' must be", as.character(out1)))
            diffs[i] <- as.numeric(NA)
        }
        else {
            rxPredict1 <- rxPredict(res1, data = iris, extraVarsToWrite = "Species")
            diffs[i] <- sum(rxPredict1$Species != rxPredict1$PredictedLabel)
        }
    }

    # conditioningConst
    # Valid ranges 0 < x < 1
    cVals <- c(-1, 0, .1, .5, 1)
    isValid <- c(FALSE, FALSE, TRUE, TRUE, FALSE)
    numVals <- length(cVals)
    diffs <- vector(mode = "numeric", length = numVals)
    for (i in seq_along(cVals)) {
        out1 <- try(
        res1 <- rxNeuralNet(formula = Species ~ Sepal.Length + Sepal.Width + Petal.Width,
            data = iris, type = "multi",
            optimizer = adaDeltaSgd(conditioningConst = cVals[i])), silent = TRUE)
        if (!isValid[i]) {
            checkTrue(inherits(out1, "try-error"))
            checkTrue(grepl("'conditioningConst' must be", as.character(out1)))
            diffs[i] <- as.numeric(NA)
        }
        else {
            rxPredict1 <- rxPredict(res1, data = iris, extraVarsToWrite = "Species")
            diffs[i] <- sum(rxPredict1$Species != rxPredict1$PredictedLabel)
        }
    }
}

"test.rxNeuralNet.args.general" <- function() {

    dataFile <- file.path(rxGetOption("sampleDataDir"), "claims.txt")

    dataColInfo <- list(
        RowNum = list(type = "integer"),
        age = list(type = "factor",
            levels = c("17-20", "21-24", "25-29", "30-34", "35-39", "40-49", "50-59", "60+")),
        car.age = list(type = "factor",
            levels = c("0-3", "4-7", "8-9", "10+")),
        type = list(type = "factor",
            levels = c("A", "B", "C", "D")),
        cost = list(type = "float32"),
        number = list(type = "float32"))

    dataSource <- RxTextData(dataFile, colInfo = dataColInfo)

    type = "regression"
    numHiddenNodes = 100
    numIterations = 100
    initWtsDiameter = 0.1

    form <- cost ~ age + car.age + type + number

    rxSetComputeContext("local")
    model <- rxNeuralNet(form, data = dataSource, type = type,
        numHiddenNodes = numHiddenNodes, numIterations = numIterations,
        initWtsDiameter = initWtsDiameter
        )
    (summ <- summary(model))
    score <- rxPredict(model, data = dataSource, extraVarsToWrite = "cost")
}

"test.rxNeuralNet.args.numHiddenNodes" <- function() {
    myIris = iris
    myIris$Setosa <- iris$Species == "setosa"

    res1 <- rxNeuralNet(formula = Setosa ~ Sepal.Length + Sepal.Width + Petal.Width,
        data = myIris,
        numHiddenNodes = 1)

    res2 <- rxNeuralNet(formula = Setosa ~ Sepal.Length + Sepal.Width + Petal.Width,
        data = myIris,
        numHiddenNodes = 10)
    sum1 <- summary(res1)
    sum2 <- summary(res2)
    checkTrue(length(sum1$summary$Weights_0) < length(sum2$summary$Weights_0))
}

"test.rxNeuralNet.args.numIterations" <- function() {
    myIris = iris
    myIris$isVersi <- iris$Species == "versicolor"

    # FIXME: Negative iterations should give error
    iterVals <- c(1, 60, 100, -1)
    numVals <- length(iterVals)
    diffs <- vector(mode = "numeric", length = numVals)

    for (i in seq_along(iterVals)) {

        res1 <- rxNeuralNet(formula = isVersi ~ Sepal.Length + Sepal.Width + Petal.Width,
            data = myIris, numIterations = iterVals[i],
            optimizer = sgd(learningRate = .10))

        rxPredict1 <- rxPredict(res1, data = myIris, extraVarsToWrite = "isVersi")
        diffs[i] <- sum(rxPredict1$isVersi != rxPredict1$PredictedLabel)
    }

    # More iterations should give better results
    checkTrue(diffs[1] > diffs[2])
    checkTrue(diffs[2] > diffs[3])

}

"test.rxNeuralNet.args.netDefinition" <- function() {
    DEACTIVATED("Test for netDefinition needs to be written.")
}

"test.rxNeuralNet.args.initWtsDiameter" <- function() {
    # Default initWtsDiameter = 0.1
    myIris = iris
    myIris$isVersi <- iris$Species == "versicolor"

    initVals <- c(-1, 0, .1, .5, 1, 2)
    numVals <- length(initVals)
    diffs <- vector(mode = "numeric", length = numVals)

    # FIXME: What are reasonable values for initWtsDiamter?
    for (i in seq_along(initVals)) {
        res1 <- rxNeuralNet(formula = isVersi ~ Sepal.Length + Sepal.Width + Petal.Width,
            data = myIris, initWtsDiameter = initVals[i],
            optimizer = sgd(learningRate = .10))

        rxPredict1 <- rxPredict(res1, data = myIris, extraVarsToWrite = "isVersi")
        diffs[i] <- sum(rxPredict1$isVersi != rxPredict1$PredictedLabel)
    }
}

"test.rxNeuralNet.args.maxNorm" <- function() {
    # Default maxNorm = 0
    myIris = iris
    myIris$isVersi <- iris$Species == "versicolor"

    initVals <- c(0, .1, .5, 1, 2)
    numVals <- length(initVals)
    diffs <- vector(mode = "numeric", length = numVals)

    for (i in seq_along(initVals)) {
        res1 <- rxNeuralNet(formula = isVersi ~ Sepal.Length + Sepal.Width + Petal.Width,
            data = myIris, maxNorm = initVals[i], numIterations = 5,
            optimizer = sgd(learningRate = .10))

        rxPredict1 <- rxPredict(res1, data = myIris, extraVarsToWrite = "isVersi")
        diffs[i] <- sum(rxPredict1$isVersi != rxPredict1$PredictedLabel)
    }

    errorOut <- try(res1 <- rxNeuralNet(formula = isVersi ~ Sepal.Length + Sepal.Width + Petal.Width,
            data = myIris, maxNorm = -1, numIterations = 5,
            optimizer = sgd(learningRate = .10)))
    checkTrue(inherits(errorOut, "try-error"))
    checkTrue(grepl("'maxNorm' must be greater than or equal to 0", as.character(errorOut)))
}

"test.rxNeuralNet.args.normalize" <- function() {

    # Quick smoke test to make sure legal values run
    modelObject1 <- rxNeuralNet(formula = Species ~ Sepal.Length + Sepal.Width + Petal.Width,
        data = iris, type = "multi")

    modelObject2 <- rxNeuralNet(formula = Species ~ Sepal.Length + Sepal.Width + Petal.Width,
        data = iris, normalize = "yes", type = "multi")

    modelObject3 <- rxNeuralNet(formula = Species ~ Sepal.Length + Sepal.Width + Petal.Width,
        data = iris, normalize = "no", type = "multi")

    modelObject4 <- rxNeuralNet(formula = Species ~ Sepal.Length + Sepal.Width + Petal.Width,
        data = iris, normalize = "auto", type = "multi")

    # Error handling: illegal value for normalize
    errorOut <- try(
        modelObject5 <- rxNeuralNet(formula = Species ~ Sepal.Length + Sepal.Width + Petal.Width,
            data = iris, type = "multi", normalize = "Hello"), silent = TRUE
        )
    checkTrue(inherits(errorOut, "try-error"))
    checkTrue(grepl("'normalize' must be one of", as.character(errorOut)))
}

"test.rxNeuralNet.args.acceleration" <- function() {
    DEACTIVATED("GPU not supported on all platforms.");
    myIris = iris
    myIris$Setosa <- iris$Species == "setosa"

    res1 <- rxNeuralNet(formula = Setosa ~ Sepal.Length + Sepal.Width + Petal.Width,
        data = myIris,
        numHiddenNodes = 10)

    res2 <- rxNeuralNet(formula = Setosa ~ Sepal.Length + Sepal.Width + Petal.Width,
        data = myIris,
        acceleration = "gpu",
        miniBatchSize = 2,
        numHiddenNodes = 10)
    sum1 <- summary(res1)
    sum2 <- summary(res2)
    checkTrue(length(sum1$weights[[1]]) == length(sum2$weights[[1]]))
    checkTrue(sum(abs(sum1$weights[[1]] - sum2$weights[[1]]) < 0.1) == 30)
}

"test.rxNeuralNet.args.hidden.randomSeed" <- function() {
    myIris = iris
    myIris$isVersi <- iris$Species == "versicolor"

    m1 <- rxNeuralNet(formula = isVersi ~ Sepal.Length + Sepal.Width + Petal.Width,
        data = myIris, numIterations = 0, randomSeed = 5)

    m2 <- rxNeuralNet(formula = isVersi ~ Sepal.Length + Sepal.Width + Petal.Width,
        data = myIris, numIterations = 0, randomSeed = 74)

    m3 <- rxNeuralNet(formula = isVersi ~ Sepal.Length + Sepal.Width + Petal.Width,
        data = myIris, numIterations = 0, randomSeed = 5)

    checkTrue(!identical(summary(m1)$summary$Weights_0, summary(m2)$summary$Weights_0))
    checkTrue(!identical(summary(m1)$summary$Weights_1, summary(m2)$summary$Weights_1))
    checkTrue(!identical(summary(m1)$summary$Biases_0, summary(m2)$summary$Biases_0))
    checkTrue(!identical(summary(m1)$summary$Biases_1, summary(m2)$summary$Biases_1))

    checkTrue(identical(summary(m1)$summary$Weights_0, summary(m3)$summary$Weights_0))
    checkTrue(identical(summary(m1)$summary$Weights_1, summary(m3)$summary$Weights_1))
    checkTrue(identical(summary(m1)$summary$Biases_0, summary(m3)$summary$Biases_0))
    checkTrue(identical(summary(m1)$summary$Biases_1, summary(m3)$summary$Biases_1))
}
"test.rxNeuralNet.MAML.code.check.optional.params" <- function() 
{
    myIris = iris
    myIris$Setosa <- iris$Species == "setosa"

    defaultMamlCode <- "lab={Setosa} feat={Sepal.Length} feat={Sepal.Width} feat={Petal.Width}   normalize=Auto trainer=BinaryNeuralNetwork{ lossFunction=CrossEntropy defaultHiddenNodes=100 numIterations=100 displayRefresh=1 optimizationAlgorithm=sgd initWtsDiameter=0.1 maxNorm=0 earlyStoppingMetrics=0 pruning=- pruningFactor=0.01 pruningRounds=10 pruningRoundIterations=5 acceleration=sse preTrainerType=NoPreTrainer miniBatchSize=1 shuffle=+ inputDropoutRate=0 hiddenDropoutRate=0} calibrator=sigmoid postTransformCache=Disk"
    allParamsChanged <- "lab={Setosa} feat={Sepal.Length} feat={Sepal.Width} feat={Petal.Width}   normalize=warn trainer=BinaryNeuralNetwork{ lossFunction=CrossEntropy defaultHiddenNodes=99 numIterations=99 displayRefresh=1 optimizationAlgorithm=ada{ decay=0.95 cond=0.000001} initWtsDiameter=0.2 maxNorm=1 earlyStoppingMetrics=0 pruning=- pruningFactor=0.01 pruningRounds=10 pruningRoundIterations=5 acceleration=sse preTrainerType=NoPreTrainer miniBatchSize=1 shuffle=+ inputDropoutRate=0 hiddenDropoutRate=0} calibrator=sigmoid postTransformCache=Disk"
    
    # Run examples
    resDefault <- rxNeuralNet(formula = Setosa ~ Sepal.Length + Sepal.Width + Petal.Width,
        data = myIris)
    
    
    resAllParamsChanged <- rxNeuralNet(formula = Setosa ~ Sepal.Length + Sepal.Width + Petal.Width,
        data = myIris, numHiddenNodes = 99, numIterations = 99, optimizer = adaDeltaSgd(), initWtsDiameter = 0.2, maxNorm = 1,
        acceleration = "sse", normalize = "warn")
    checkEquals(resDefault$mamlCode, defaultMamlCode)
    checkEquals(resAllParamsChanged$mamlCode, allParamsChanged)
}

"test.rxNeuralNet.ensemble.binary" <- function()
{
    #library(MicrosoftML)
    #rm(list=ls())

    myDf <- within(iris, {Setosa <- (Species == "setosa")})
    myFormula <- Setosa ~ Sepal.Length + Sepal.Width + Petal.Length + Sepal.Width + Species

    randomSeed <- 111
    replace <- TRUE
    sampRate <- 0.75
    combineMethod <- NULL

    # data frame
    modelCount <- 1
    resDf <- rxNeuralNet(formula = myFormula, data = myDf
        , ensemble = ensembleControl(modelCount = modelCount, replace = replace)
        )
    checkTrue(!inherits(resDf, "Ensemble"))
    #print(structure(resDf$mamlCode, class = "maml"))

    # xdf file
    myXdf <- tempfile(pattern = "myXdf", fileext = ".xdf")
    on.exit(unlink(myXdf), add = TRUE)
    rxDataStep(myDf, myXdf, rowsPerRead = 50, overwrite = TRUE)
    myXdfDS <- RxXdfData(file = myXdf)

    modelCount <- 2
    resXdf <- rxNeuralNet(formula = myFormula, data = myXdfDS
        , ensemble = ensembleControl(modelCount = modelCount, sampRate = sampRate)
        )
    checkTrue(inherits(resXdf, "Ensemble"))
    #cat(resXdf$mamlCode, "\n")

    # csv
    myCsv <- tempfile(pattern = "myCsv", fileext = ".csv")
    on.exit(unlink(myCsv), add = TRUE)
    myCsvDS <- RxTextData(file = myCsv, colInfo = rxCreateColInfo(iris))
    rxDataStep(myDf, myCsvDS, overwrite = TRUE)

    modelCount <- 3
    resCsv <- rxNeuralNet(formula = myFormula, data = myCsvDS
        , ensemble = ensembleControl(modelCount = modelCount, combineMethod = combineMethod, randomSeed = randomSeed)
        )
    checkTrue(inherits(resCsv, "Ensemble"))
    #cat(resCsv$mamlCode, "\n")
}

"test.rxNeuralNet.ensemble.regression" <- function()
{
    #library(MicrosoftML)
    #rm(list=ls())

    myDf <- within(iris, {Setosa <- (Species == "setosa")})
    myFormula <- Sepal.Length ~ Sepal.Width + Petal.Length + Sepal.Width + Species

    randomSeed <- 111
    replace <- TRUE
    sampRate <- 0.75
    combineMethod <- NULL
    type <- "regression"

    # data frame
    modelCount <- 1
    resDf <- rxNeuralNet(formula = myFormula, data = myDf, type = type
        , ensemble = ensembleControl(modelCount = modelCount, replace = replace)
        )
    checkTrue(!inherits(resDf, "Ensemble"))
    #print(structure(resDf$mamlCode, class = "maml"))

    # xdf file
    myXdf <- tempfile(pattern = "myXdf", fileext = ".xdf")
    on.exit(unlink(myXdf), add = TRUE)
    rxDataStep(myDf, myXdf, rowsPerRead = 50, overwrite = TRUE)
    myXdfDS <- RxXdfData(file = myXdf)

    modelCount <- 2
    resXdf <- rxNeuralNet(formula = myFormula, data = myXdfDS, type = type
        , ensemble = ensembleControl(modelCount = modelCount, sampRate = sampRate, randomSeed = randomSeed)
        )
    checkTrue(inherits(resXdf, "Ensemble"))
    #cat(resXdf$mamlCode, "\n")

    # csv
    myCsv <- tempfile(pattern = "myCsv", fileext = ".csv")
    on.exit(unlink(myCsv), add = TRUE)
    myCsvDS <- RxTextData(file = myCsv, colInfo = rxCreateColInfo(iris))
    rxDataStep(myDf, myCsvDS, overwrite = TRUE)

    modelCount <- 3
    resCsv <- rxNeuralNet(formula = myFormula, data = myCsvDS, type = type
        , ensemble = ensembleControl(modelCount = modelCount, combineMethod = combineMethod, randomSeed = randomSeed)
        )
    checkTrue(inherits(resCsv, "Ensemble"))
    #cat(resCsv$mamlCode, "\n")
}

"test.rxNeuralNet.ensemble.multiClass" <- function()
{
    #library(MicrosoftML)
    #rm(list=ls())

    myDf <- within(iris, {Setosa <- (Species == "setosa")})
    myFormula <- Species ~ Sepal.Length + Sepal.Width + Petal.Length + Sepal.Width + Setosa

    randomSeed <- 111
    replace <- TRUE
    sampRate <- 0.75
    combineMethod <- NULL
    type <- "multiClass"

    # data frame
    modelCount <- 1
    resDf <- rxNeuralNet(formula = myFormula, data = myDf, type = type
        , ensemble = ensembleControl(modelCount = modelCount, replace = replace)
        )
    checkTrue(!inherits(resDf, "Ensemble"))
    #print(structure(resDf$mamlCode, class = "maml"))

    # xdf file
    myXdf <- tempfile(pattern = "myXdf", fileext = ".xdf")
    on.exit(unlink(myXdf), add = TRUE)
    rxDataStep(myDf, myXdf, rowsPerRead = 50, overwrite = TRUE)
    myXdfDS <- RxXdfData(file = myXdf)

    modelCount <- 2
    resXdf <- rxNeuralNet(formula = myFormula, data = myXdfDS, type = type
        , ensemble = ensembleControl(modelCount = modelCount, sampRate = sampRate, randomSeed = randomSeed)
        )
    checkTrue(inherits(resXdf, "Ensemble"))
    #cat(resXdf$mamlCode, "\n")

    # csv
    myCsv <- tempfile(pattern = "myCsv", fileext = ".csv")
    on.exit(unlink(myCsv), add = TRUE)
    myCsvDS <- RxTextData(file = myCsv, colInfo = rxCreateColInfo(iris))
    rxDataStep(myDf, myCsvDS, overwrite = TRUE)

    modelCount <- 3
    resCsv <- rxNeuralNet(formula = myFormula, data = myCsvDS, type = type
        , ensemble = ensembleControl(modelCount = modelCount, combineMethod = combineMethod, randomSeed = randomSeed)
        )
    checkTrue(inherits(resCsv, "Ensemble"))
    #cat(resCsv$mamlCode, "\n")
}
