# Copyright (c) Microsoft Corporation.  All rights reserved.

# Tests for basic functionality of utility functions:
# - rxPredict

".rxSqliteConnectionStringFromFile" <- function()
{
    if (exists("rxSqliteConnectionStringFromFile", globalenv()))
        get("rxSqliteConnectionStringFromFile", globalenv())
    else
        RevoScaleR:::rxSqliteConnectionStringFromFile
}

"test.rxPredict.basic" <- function()
{
    textOutFile <- tempfile(pattern = "logitScore", fileext = ".txt")
    if (file.exists(textOutFile)) file.remove(textOutFile)
    # Estimate a logistic regression model
    infert1 <- infert
    infert1$isCase = (infert1$case == 1)    
    lrObj <- rxLogisticRegression(formula = isCase ~ age + parity + education + spontaneous + induced,
                           data = infert1)
                       
    # Put the per-instance results in a text file
    scoreDS <- rxPredict( lrObj, 
         data = infert1, extraVarsToWrite = c("isCase", "Probability"),
         outData = textOutFile, overwrite = TRUE)

    # Use the new data source to create an ROC curve
    res1 <- rxRoc(actualVarName = "isCase", predVarNames = "Probability", data = scoreDS)
    
    rxLogitObj <-rxLogit(formula = isCase ~ age + parity + education + spontaneous + induced,
                           data = infert1)
    predOut <- rxPredict(rxLogitObj, data = infert1, extraVarsToWrite = "isCase")
    res2 <- rxRoc(actualVarName = "isCase", predVarNames = "isCase_Pred", data = predOut)
    
    auc1 <- rxAuc(res1)
    auc2 <- rxAuc(res2)
    # Check to see if the two AUC's are in the same ballpark
    checkEquals(auc1, auc2, tolerance = .05)
    file.remove(textOutFile)
}

"test.rxPredict.args.extraVarsToWrite" <- function()
{
    myIris <- iris
    myIris$SpeciesStr <- as.character(iris$Species)
    myIris$Setosa <- iris$Species == "setosa"
    res1 <- rxLogisticRegression(formula = Setosa~Sepal.Length + Sepal.Width + Petal.Width,
           data = myIris)
    
    # Output character data
    scoreData1 <- rxPredict(res1, data = myIris, outData = NULL, extraVarsToWrite = c("SpeciesStr", "Setosa"))
    checkEquals(nrow(scoreData1), 150)
    checkEquals(scoreData1$Setosa, scoreData1$PredictedLabel)
    checkEquals(scoreData1$Setosa, myIris$Setosa)
    checkEquals(scoreData1$SpeciesStr, myIris$SpeciesStr)
    checkTrue(!anyNA(scoreData1$Setosa))
    
    # We don't have proper handling for non existing files which leads to segpipe and R process corruption.
    # There is two option to solve this problem. in RevoScaleR they call rxGetOutDataSource in rxPredict to check presence of file.
    # Another option is properly catch exception and don't get segpipe.
    if (.Platform$OS=="windows")
    {
    # Error handling 
    errorOut <- try(  
        scoreData1 <- rxPredict(res1, data = myIris, outData = file.path(tempdir(), "NoDir", "NoFile.xdf"), extraVarsToWrite = c("Setosa"))
       )
    checkTrue(inherits(errorOut, "try-error"))    
    }
    
    # Output factor data
    res2 <- rxLogisticRegression(formula = Species~Sepal.Length + Sepal.Width + Petal.Width,
       type = "multi", data = iris)
    scoreData2 <- rxPredict(res2, data = iris, outData = NULL, extraVarsToWrite = "Species")
    checkEquals(nrow(scoreData2), 150)
    checkEquals(scoreData2$Species, iris$Species)
    
    # Use transformVars
    outVars3 <- c("Species", "Sepal.Width", "isVersi", "Sepal.Length")
    res3 <- rxLogisticRegression(formula = Species~Sepal.Length + Sepal.Width + Petal.Width,
       type = "multi", data = iris,  transforms = list(isVersi = Species == "versicolor"))
    scoreData3 <- rxPredict(res3, data = iris, outData = NULL, 
        extraVarsToWrite = outVars3)    
    checkEquals(names(scoreData3)[1:4], outVars3) 
    
    # Same with .xdf file
    xdfInFile <- tempfile(pattern = "irisChunk1", fileext = ".xdf")
    on.exit(file.remove(xdfInFile), add = TRUE)
    
    xdfScoreFile <- tempfile(pattern = "irisScore1", fileext = ".xdf")
    on.exit(file.remove(xdfScoreFile), add = TRUE)   
    
    rxDataStep(iris, xdfInFile, rowsPerRead = 60, overwrite = TRUE) 
    
    res4 <- rxLogisticRegression(formula = Species~Sepal.Length + Sepal.Width + Petal.Width,
       type = "multi", data = xdfInFile,  transforms = list(isVersi = Species == "versicolor"))
    scoreData4 <- rxPredict(res4, data = xdfInFile, outData = xdfScoreFile, 
        extraVarsToWrite = outVars3)  
    scoreData4 <- rxDataStep(scoreData4)  
    checkEquals(scoreData3, scoreData4) 
    checkEquals(nrow(scoreData4), 150)    
     
}

"test.rxPredict.args.extraVarsToWrite.raw" <- function()
{
    DEACTIVATED("Bug 63979: test.rxPredict.args.extraVarsToWrite.raw seems to fail intermittently on Linux plaforms.")
    #library(MicrosoftML)

    ## setup data
    xdf <- file.path(rxGetOption("sampleDataDir"), "claims4blocks.xdf")
    rdf <- rxDataStep(xdf)
    #rxGetInfo(rdf, getVarInfo = TRUE, getBlockSizes = TRUE)

    ## create a variable of VarType_String
    rdf[["string"]] <- as.character(rdf[["age"]])

    ## create a variable of VarType_FixedRaw
    ## NOTE: .rxColWidth must be an integer (i.e., a number ending with 'L') for this var to be treated as FixedRaw!
    rdf[["fixed.raw"]] <- structure(as.character(rdf[["car.age"]]), .rxColWidth = 20L)

    ## create a variable of VarType_VariableRaw
    raws <- charToRaw(paste(letters, collapse = ""))
    rdf[["variable.raw"]] <- I(sapply(seq(nrow(rdf)), function(i, r) r[seq(i%%length(letters))], raws))

    head(rdf)

    ## setup modeling
    form <- cost ~ number + type
    extraVarsToWrite <- c("string", "fixed.raw", "variable.raw")

    ## mml
    model.mml <- rxFastTrees(form, data = rdf, type = "regression")     #rxPredict Error
    pred.mml <- rxPredict(model.mml, data = rdf, overwrite = TRUE
        , extraVarsToWrite = extraVarsToWrite
        )
    head(pred.mml)

    ## revo
    model.revo <- rxBTrees(form, data = rdf, lossFunction = "gaussian", nTree = 2, maxDepth = 1)    #rxPredict OK
    pred.revo <- rxPredict(model.revo, data = rdf, overwrite = TRUE
        , extraVarsToWrite = extraVarsToWrite
        )
    head(pred.revo)

    ## check
    vars <- c("string", "fixed.raw", "variable.raw")
    checkEquals(pred.mml[vars], pred.revo[vars])

    attrs.data <- sapply(rdf[vars], attributes)
    attrs.pred <- sapply(pred.mml[vars], attributes)
    #checkEquals(attrs.data, attrs.pred) #variable.raw is "AsIs" in data but "ODBC_binary" in pred.
    checkEquals(attrs.data[["fixed.raw"]], attrs.pred[["fixed.raw"]])
}

"test.rxPredict.args.writeModelVars" <- function()
{
    DEACTIVATED("Defect 43217:  Investigate and reenable test")
    # Use transformVars
    outVars1 <- c("Species", "Sepal.Width", "isVersi", "Sepal.Length")
	myForm <- isVersi~Sepal.Length + Petal.Width
    
    # With writeModelVars
    res1 <- rxLogisticRegression(formula = myForm,
        data = iris,  transforms = list(isVersi = Species == "versicolor"))
    scoreData1 <- rxPredict(res1, data = iris, outData = NULL, 
        writeModelVars = TRUE, extraVarsToWrite = outVars1)    
	scoreData1$Score <- NULL
	scoreData1$PredictedLabel <- NULL
	
	res2 <- rxLogit(formula = myForm, data = iris,
        transforms = list(isVersi = Species == "versicolor"))
	predData2 <- rxPredict(res2, data = iris, outData = NULL, writeModelVars = TRUE,
        #extraVarsToWrite = outVars1) #TODO (lgong) 2017-04-05: a regression was introduced by * Merged PR 24787: Throw and error if extraVarsToWrite is not present in input data (3/29)
		extraVarsToWrite = setdiff(outVars1, "isVersi"))
	predNames <- names(predData2)
	predNames <- sub("isVersi_Pred", "Probability", predNames)
	checkEquals(sort(predNames), sort(names(scoreData1))) 
    
    # Without writeModelVars
    scoreData2 <- rxPredict(res1, data = iris, outData = NULL, 
        extraVarsToWrite = outVars1)      
    checkEquals(names(scoreData2)[1:4], outVars1)
    
    # Without extraVarsToWrite
    scoreData3 <- rxPredict(res1, data = iris, outData = NULL, 
        writeModelVars = TRUE)      
    checkEquals(names(scoreData3)[1:4], c("isVersi", "Sepal.Length", "Petal.Width", "Species")) 
 
}

"test.rxPredict.args.suffix" <- function()
{
	myForm <- PredictedLabel~Petal.Length + Petal.Width
    
    myIris <- iris
    myIris$PredictedLabel = myIris$Species == "versicolor"
    myIris$isVersi <- myIris$PredictedLabel
    # With writeModelVars
    res1 <- rxLogisticRegression(formula = myForm,
        data = myIris)
    scoreData1 <- rxPredict(res1, data = myIris, outData = NULL, 
        writeModelVars = TRUE, extraVarsToWrite = "isVersi", suffix = "Pred") 
    
    checkEquals(scoreData1$isVersi, scoreData1$PredictedLabel)  
	checkTrue("ProbabilityPred" %in% names(scoreData1))   
}

"test.rxPredict.args.outData.xdf.chunkSize" <- function()
{
    xdfInFile <- tempfile(pattern = "irisChunk2", fileext = ".xdf")
    on.exit(file.remove(xdfInFile), add = TRUE)
    
    xdfScoreFile <- tempfile(pattern = "irisScore2", fileext = ".xdf")
    on.exit(file.remove(xdfScoreFile), add = TRUE)
    
    csvInFile <- tempfile(pattern = "myIris2", fileext = ".csv")
    on.exit(file.remove(csvInFile), add = TRUE) 
        
    myIris = iris
    myIris$Setosa <- iris$Species == "setosa"
        
    # Create xdf file with differing block sizes
    rxDataStep(myIris, xdfInFile, rowsPerRead = 60, overwrite = TRUE)
    rxDataStep(myIris, xdfInFile, rowsPerRead = 26, append="rows")
    fileInfoIn <- rxGetInfo(xdfInFile, getBlockSizes = TRUE)

    outModel1 <- rxLogisticRegression(formula = Setosa~Sepal.Length + Sepal.Width + Petal.Width,
        data = xdfInFile)
         
    outDS <- rxPredict(outModel1, data = xdfInFile, outData = xdfScoreFile, overwrite = TRUE)

    fileInfoScore <- rxGetInfo(outDS, getBlockSizes = TRUE)
    
    checkEquals(fileInfoIn$numBlocks, fileInfoScore$numBlocks)
    checkEquals(fileInfoIn$rowsPerBlock, fileInfoScore$rowsPerBlock)
    
    # Now use a csv input data file
    testNumRows <- 40
    csvInDS <- RxTextData(file = csvInFile, rowsPerRead = testNumRows)
    rxDataStep(myIris, csvInDS, overwrite = TRUE)
        
    outModel1 <- rxLogisticRegression(formula = Setosa~Sepal.Length + Sepal.Width + Petal.Width,
        data = csvInDS)
         
    outDS2 <- rxPredict(outModel1, data = csvInDS, outData = xdfScoreFile, overwrite = TRUE)
    fileInfoScore2 <- rxGetInfo(outDS2, getBlockSizes = TRUE) 
    if (rxIsExpressEdition())
    {
        # Express Edition ignores rowsPerRead
        checkEquals(fileInfoScore2$rowsPerBlock[1],nrow(iris)) 
        checkEquals(fileInfoScore2$numBlocks, 1)         
    } else
    { 
        checkEquals(fileInfoScore2$rowsPerBlock[1], testNumRows) 
        checkEquals(fileInfoScore2$numBlocks, 4) 
    }    
}

"test.rxPredict.args.reportProgress" <- function()
{
    infert1 <- infert
    infert1$isCase = (infert1$case == 1)    
    lrObj <- rxLogisticRegression(formula = isCase ~ age + parity + education + spontaneous + induced,
                           data = infert1)
                       
    out0 <- capture.output(scoreDF <- rxPredict( lrObj, 
         data = infert1, extraVarsToWrite = c("isCase", "Probability"),
         reportProgress = 0, verbose = 0))
    checkEquals(length(out0), 0)
    
    out1 <- capture.output(scoreDF <- rxPredict( lrObj, 
         data = infert1, extraVarsToWrite = c("isCase", "Probability"),
         reportProgress = 1, verbose = 0))
    checkTrue(length(out1) > 0)  
    
    out2 <- capture.output(scoreDF <- rxPredict( lrObj, 
         data = infert1, extraVarsToWrite = c("isCase", "Probability"),
         reportProgress = 2, verbose = 0))
    out2str <- paste(out2, collapse = "")
    out1str  <- paste(out1, collapse = "")
    checkTrue(nchar(out2str) > nchar(out1str))  
}

"test.rxPredict.args.blocksPerRead" <- function()
{
    xdfInFile <- tempfile(pattern = "irisChunk3", fileext = ".xdf")
    on.exit(file.remove(xdfInFile), add = TRUE)
    
    xdfScoreFile <- tempfile(pattern = "irisScore3", fileext = ".xdf")
    on.exit(file.remove(xdfScoreFile), add = TRUE)
           
    myIris = iris
    myIris$isVersi <- iris$Species == "versicolor"
        
    # Create xdf file with differing block sizes
    rxDataStep(myIris, xdfInFile, rowsPerRead = 60, overwrite = TRUE)
    rxDataStep(myIris, xdfInFile, rowsPerRead = 26, append="rows")
    fileInfoIn <- rxGetInfo(xdfInFile, getBlockSizes = TRUE)

    outModel1 <- rxLogisticRegression(formula = isVersi~Sepal.Length + Sepal.Width + Petal.Width,
        data = xdfInFile)
         
    out1 <- capture.output(
        outDS <- rxPredict(outModel1, data = xdfInFile, outData = xdfScoreFile, 
            blocksPerRead = 1, overwrite = TRUE)
        )
    
    out2 <- capture.output(
        outDS <- rxPredict(outModel1, data = xdfInFile, outData = xdfScoreFile, 
            blocksPerRead = 2, overwrite = TRUE)
        )
    
    if (rxIsExpressEdition()) {
        checkTrue(length(out1) == length(out2))
	} else {
		checkTrue(length(out1) > length(out2))
	}
}

"test.rxPredict.classification.integer.label" <- function()
{
    DEACTIVATED("Bug 14543: rxPredict should not require response source column for classification models trained on integral labels.")
    
    ## multiClass
    iris3 <- iris    
    resp3 <- "SpeciesId"
    #iris3[[resp3]] <- iris3$Species #factor OK
    iris3[[resp3]] <- as.integer(iris3$Species) #integer failed rxPredict
    #iris3[[resp3]] <- as.double(iris3$Species) #double failed rxPredict
    form3 <- paste(resp3, "Sepal.Length + Sepal.Width + Petal.Length + Petal.Width", sep = " ~ ")
    type3 <- "multiClass"
    
    logitModel3 <- rxLogisticRegression(form3, data = iris3, type = type3)
    logitPred3 <- rxPredict(logitModel3, data = iris)
    
    nnetModel3 <- rxNeuralNet(form3, data = iris3, type = type3)
    nnetPred3 <- rxPredict(nnetModel3, data = iris)
    
    ## binary
    iris2 <- iris    
    resp2 <- "IsVersi"
    #iris2[[resp2]] <- (iris2$Species == "versicolor") #logical OK
    #iris2[[resp2]] <- as.factor(iris2$Species == "versicolor") #factor OK
    iris2[[resp2]] <- as.integer(iris2$Species == "versicolor") #integer failed rxPredict
    #iris2[[resp2]] <- as.double(iris2$Species == "versicolor") #double failed rxPredict
    form2 <- paste(resp2, "Sepal.Length + Sepal.Width + Petal.Length + Petal.Width", sep = " ~ ")
    type2 <- "binary"
    
    logitModel2 <- rxLogisticRegression(form2, data = iris2, type = type2)
    logitPred2 <- rxPredict(logitModel2, data = iris)
    
    nnetModel2 <- rxNeuralNet(form2, data = iris2, type = type2)
    nnetPred2 <- rxPredict(nnetModel2, data = iris)
    
    forestModel2 <- rxFastForest(form2, data = iris2, type = type2)
    forestPred2 <- rxPredict(forestModel2, data = iris)
    
    treeModel2 <- rxFastTrees(form2, data = iris2, type = type2)
    treePred2 <- rxPredict(treeModel2, data = iris)
    
    linearModel2 <- rxFastLinear(form2, data = iris2, type = type2)
    linearPred2 <- rxPredict(linearModel2, data = iris)
    
    checkTrue(TRUE)
}


"test.rxPredict.regression.non-float32.label" <- function()
{   
    DEACTIVATED("Bug 32499: rxPredict should not require response source column for regression models trained on non-float32 labels.")

    ## train data
    dataFile <- file.path(rxGetOption("sampleDataDir"), "claims.txt")

    dataColInfo <- list(
        RowNum = list(type = "integer"),
        age = list(type = "factor",
            levels = c("17-20", "21-24", "25-29", "30-34", "35-39", "40-49", "50-59", "60+")),
        car.age = list(type = "factor",
            levels = c("0-3", "4-7", "8-9", "10+")),
        type = list(type = "factor",
            levels = c("A", "B", "C", "D")),
		cost = list(type = "float64"),  #any type for regression response variable other than "float32" (which corresponds to R4 in TLC) fails rxPredict.
		number = list(type = "float32"))
	
	dataSource <- RxTextData(dataFile, colInfo = dataColInfo)
	
    ## train model
	type <- "regression"
    resp <- "cost"
	form <- formula(paste(resp, "age + car.age + type + number", sep = " ~ "))
    model <- rxFastTrees(form, data = dataSource, type = type)

    ## test data
    outDataFile <- tempfile(fileext = ".txt")
    on.exit(if(file.exists(outDataFile)) unlink(outDataFile), add = TRUE)
	outDataSource <- RxTextData(outDataFile, colInfo = dataColInfo)
    rxDataStep(dataSource, outDataSource, overwrite = TRUE, varsToDrop = resp)

    ## check prediction
    score <- try(rxPredict(model, data = outDataSource))
    checkTrue(!inherits(score, "try-error"))

    ## regression
    iris1 <- iris    
    resp1 <- "SpeciesInt"
    iris1[[resp1]] <- as.integer(iris2$Species) #integer failed rxPredict
    form1 <- paste(resp1, "Sepal.Length + Sepal.Width + Petal.Length + Petal.Width", sep = " ~ ")
    type1 <- "regression"
    
    nnetModel1 <- rxNeuralNet(form1, data = iris1, type = type1)
    nnetPred1 <- rxPredict(nnetModel1, data = iris)
    
    forestModel1 <- rxFastForest(form1, data = iris1, type = type1)
    forestPred1 <- rxPredict(forestModel1, data = iris)
    
    treeModel1 <- rxFastTrees(form1, data = iris1, type = type1)
    treePred1 <- rxPredict(treeModel1, data = iris)
    
    linearModel1 <- rxFastLinear(form1, data = iris1, type = type1)
    linearPred1 <- rxPredict(linearModel1, data = iris)

    checkTrue(TRUE)
}

"test.rxFeaturize.scalar" <- function()
{
    claimsXdf <- file.path(rxGetOption("sampleDataDir"), "claims.xdf")
    claimsDF <- rxDataStep(inData = claimsXdf)   
    claimsDF$ageStr <- as.character(claimsDF$age)
    claimsDF$typeStr <- as.character(claimsDF$type)

    scalar.cols <- c("RowNum", "age", "car.age", "type", "cost", "number", "ageStr", "typeStr")
    vector.cols <- c("str", "cat")

    features.select.scalar <- rxFeaturize(data = claimsDF, 
        mlTransforms = list(
            concat(vars = list(str = c("ageStr", "typeStr")))
            , categoricalHash(vars = c(cat = "str"))
            , selectColumns(scalar.cols)
        ))

    features.drop.vector1 <- rxFeaturize(data = claimsDF, 
        mlTransforms = list(
            concat(vars = list(str = c("ageStr", "typeStr")))
            , categoricalHash(vars = c(cat = "str"))
            , dropColumns(vector.cols)
        ), mlTransformVars = scalar.cols)   #need to know the original vars due to the lack of formula

    checkIdentical(features.select.scalar, features.drop.vector1[names(features.select.scalar)])

    features.drop.vector2 <- rxFeaturize(data = claimsDF, 
        mlTransforms = list(
            concat(vars = list(str = c("ageStr", "typeStr")))
            , categoricalHash(vars = c(cat = "str"))
            , dropColumns(vector.cols)
        ), transformVars = scalar.cols)     #need to know the original vars due to the lack of formula

    checkIdentical(features.drop.vector1, features.drop.vector2[names(features.drop.vector1)])

    features.select.drop <- MicrosoftML:::rxFeaturize(data = claimsDF, 
        mlTransforms = list(
            concat(vars = list(str = c("ageStr", "typeStr")))
            , categoricalHash(vars = c(cat = "str"))
            , selectColumns(scalar.cols)
            , dropColumns(vector.cols)
        ))

    checkIdentical(features.drop.vector2, features.select.drop[names(features.drop.vector2)])
}

"test.rxFeaturize.vector" <- function()
{
    #DEACTIVATED("Bug 37662: outputing vector columns failed.")

    claimsXdf <- file.path(rxGetOption("sampleDataDir"), "claims.xdf")
    claimsDF <- rxDataStep(inData = claimsXdf)   
    claimsDF$ageStr <- as.character(claimsDF$age)
    claimsDF$typeStr <- as.character(claimsDF$type)

    scalar.cols <- c("RowNum", "age", "car.age", "type", "cost", "number", "ageStr", "typeStr")
    vector.cols <- c("str", "cat")

    nrows <- nrow(claimsDF)
    ncols <- ncol(claimsDF)

    ## use default settings
    features.default <- rxFeaturize(data = claimsDF, 
        mlTransforms = list(
            concat(vars = list(str = c("ageStr", "typeStr")))
            , categoricalHash(vars = c(cat = "str"))
        ))
    checkEquals(dim(features.default), c(nrows, 5004))

    ## set hashBits
    features.hashBits <- rxFeaturize(data = claimsDF, 
        mlTransforms = list(
            concat(vars = list(str = c("ageStr", "typeStr")))
            , categoricalHash(vars = c(cat = "str"), hashBits = 4)
        ))
    checkEquals(dim(features.hashBits), c(nrows, 20))

    ## set maxSlots
    features.maxSlots <- rxFeaturize(data = claimsDF, 
        mlTransforms = list(
            concat(vars = list(str = c("ageStr", "typeStr")))
            , categoricalHash(vars = c(cat = "str"))
        ), maxSlots = 4)
    checkEquals(dim(features.maxSlots), c(nrows, 8))

    ## select solumns
    features.select <- rxFeaturize(data = claimsDF, 
        mlTransforms = list(
            concat(vars = list(str = c("ageStr", "typeStr")))
            , categoricalHash(vars = c(cat = "str"))
            , selectColumns(c(scalar.cols, vector.cols))
        ))
    checkEquals(dim(features.select), c(nrows, 5010))
}

"test.rxFeaturize.image" <- function()
{
    train <- data.frame(
        Path = c(
            system.file("help/figures/RevolutionAnalyticslogo.png", package = "MicrosoftML"),
            # REVIEW coeseanu: Seems bmps are not included with the package?!
            # system.file("help/figures/RevolutionAnalyticslogo.bmp", package = "MicrosoftML"),
            system.file("help/figures/RevolutionAnalyticslogo.jpg", package = "MicrosoftML")
        ), 
        Label = c(1, 1), 
        stringsAsFactors = FALSE)

    scores <- rxFeaturize(
        data = train,
        mlTransforms = list(
            loadImage(vars = list(Features = "Path")),
            resizeImage(vars = "Features", width = 1, height = 1, resizing = "Aniso"),
            extractPixels(vars = "Features")
            ),
        mlTransformVars = "Path"
        )

    checkEquals(scores$Features.0, c(255, 254))
    checkEquals(scores$Features.1, c(255, 254))
    checkEquals(scores$Features.2, c(255, 254))
}

"test.rxFeaturize.text.wordFeatureExtractor.ngramCount" <- function()
{
    #library(MicrosoftML)
    trainReviews <- data.frame(opinion = c(
        "I love it",
        "I love it",
        "Love it",
        "I love it a lot",
        "Really love it",
        "I love it",
        "I love it",
        "love it",
        "I love it a lot",
        "Really love it",
        "I hate it",
        "I hate it",
        "I hate it",
        "hate it",
        "hate and",
        "I hate it and",
        "I hate it and",
        "I hate it and",
        "hate it and",
        "hate and"),
        like = rep(c(TRUE, FALSE), each = 10),
        stringsAsFactors = FALSE)

    #NOTE (lgong) 2017-04-04: the default "auto" behaves differently for different data input
    normalize <- "warn" #"auto" #"no"   #"yes"  #

    ## train directly with mlTransforms
    outModel.wordcount1 <- rxLogisticRegression(like ~ features, data = trainReviews,
        l2Weight = 1, l1Weight = 1, optTol = 1e-07, memorySize = 20,
        initWtsScale = 0, maxIterations = 2147483647, showTrainingStats = FALSE,
        sgdInitTol = 0, trainThreads = NULL, denseOptimizer = FALSE, 
        normalize = normalize, 
        mlTransforms = list(featurizeText(
            vars = list(features = "opinion")
            , language = "English"
            , stopwordsRemover = NULL   #ignored
            , case = "lower"
            , keepDiacritics = FALSE
            , keepPunctuations = TRUE
            , keepNumbers = TRUE
            , dictionary = NULL         #ignored
            , wordFeatureExtractor = ngramCount()
            , charFeatureExtractor = NULL
            , vectorNormalizer = "l2"
           )
        )
    )
    coef(outModel.wordcount1)
    #MicrosoftML:::print.maml(outModel.wordcount1$mamlCode)

    ## featurize first and then train
    ftext.wordcount <- rxFeaturize(data = trainReviews,
        dataThreads = NULL, randomSeed = NULL, maxSlots = 5000, 
        mlTransforms = list(featurizeText(
            vars = list(features = "opinion")
            , language = "English"
            , stopwordsRemover = NULL   #ignored
            , case = "lower"
            , keepDiacritics = FALSE
            , keepPunctuations = TRUE
            , keepNumbers = TRUE
            , dictionary = NULL         #ignored
            , wordFeatureExtractor = ngramCount()
            , charFeatureExtractor = NULL
            , vectorNormalizer = "l2"
            )
        )
    )

    allvars <- colnames(ftext.wordcount)
    yvar <- "like"
    tvar <- "opinion"
    xvars <- setdiff(allvars, c(yvar, tvar))
    form <- paste(yvar, paste(xvars, collapse = " + "), sep  = " ~ ")

    #outModel.wordcount2 <- rxLogisticRegression(form, data = ftext.wordcount,
        #l2Weight = 1, l1Weight = 1, optTol = 1e-07, memorySize = 20,
        #initWtsScale = 0, maxIterations = 2147483647, showTrainingStats = FALSE,
        #sgdInitTol = 0, trainThreads = NULL, denseOptimizer = FALSE, 
        #normalize = normalize
    #)
    #coef(outModel.wordcount2)
    ##MicrosoftML:::print.maml(outModel.wordcount2$mamlCode)

    ### check
    #checkEquals(coef(outModel.wordcount1), coef(outModel.wordcount2), checkNames = FALSE)
}

"test.rxFeaturize.text.wordFeatureExtractor.ngramHash" <- function()
{
    #library(MicrosoftML)
    trainReviews <- data.frame(opinion = c(
        "I love it",
        "I love it",
        "Love it",
        "I love it a lot",
        "Really love it",
        "I love it",
        "I love it",
        "love it",
        "I love it a lot",
        "Really love it",
        "I hate it",
        "I hate it",
        "I hate it",
        "hate it",
        "hate and",
        "I hate it and",
        "I hate it and",
        "I hate it and",
        "hate it and",
        "hate and"),
        like = rep(c(TRUE, FALSE), each = 10),
        stringsAsFactors = FALSE)

    #NOTE (lgong) 2017-04-04: the default "auto" behaves differently for different data input
    normalize <- "warn" #"auto" #"no"   #"yes"  #

    #TODO (lgong) 2017-04-04: why the results are different when hashBits is greater than 12?
    hashBits <- 12
    seed <- 314489979

    ## train directly with mlTransforms
    outModel.wordhash1 <- rxLogisticRegression(like ~ features, data = trainReviews,
        l2Weight = 1, l1Weight = 1, optTol = 1e-07, memorySize = 20,
        initWtsScale = 0, maxIterations = 2147483647, showTrainingStats = FALSE,
        sgdInitTol = 0, trainThreads = NULL, denseOptimizer = FALSE, 
        normalize = normalize, 
        mlTransforms = list(featurizeText(
            vars = list(features = "opinion")
            , language = "English"
            , stopwordsRemover = NULL   #ignored
            , case = "lower"
            , keepDiacritics = FALSE
            , keepPunctuations = TRUE
            , keepNumbers = TRUE
            , dictionary = NULL         #ignored
            , wordFeatureExtractor = ngramHash(hashBits = hashBits, seed = seed)
            , charFeatureExtractor = NULL
            , vectorNormalizer = "l2"
           )
        )
    )
    coef(outModel.wordhash1)
    #MicrosoftML:::print.maml(outModel.wordhash1$mamlCode)

    ## featurize first and then train
    ftext.wordhash <- rxFeaturize(data = trainReviews,
        dataThreads = NULL, randomSeed = NULL, maxSlots = 5000, 
        mlTransforms = list(featurizeText(
            vars = list(features = "opinion")
            , language = "English"
            , stopwordsRemover = NULL   #ignored
            , case = "lower"
            , keepDiacritics = FALSE
            , keepPunctuations = TRUE
            , keepNumbers = TRUE
            , dictionary = NULL         #ignored
            , wordFeatureExtractor = ngramHash(hashBits = hashBits, seed = seed)
            , charFeatureExtractor = NULL
            , vectorNormalizer = "l2"
            )
        )
    )

    allvars <- colnames(ftext.wordhash)
    yvar <- "like"
    tvar <- "opinion"
    xvars <- setdiff(allvars, c(yvar, tvar))
    form <- paste(yvar, paste(xvars, collapse = " + "), sep  = " ~ ")

    #outModel.wordhash2 <- rxLogisticRegression(form, data = ftext.wordhash,
        #l2Weight = 1, l1Weight = 1, optTol = 1e-07, memorySize = 20,
        #initWtsScale = 0, maxIterations = 2147483647, showTrainingStats = FALSE,
        #sgdInitTol = 0, trainThreads = NULL, denseOptimizer = FALSE, 
        #normalize = normalize
    #)
    #coef(outModel.wordhash2)
    ##MicrosoftML:::print.maml(outModel.wordhash2$mamlCode)

    ### check
    #checkEquals(coef(outModel.wordhash1), coef(outModel.wordhash2), checkNames = FALSE, tolerance = 1e-5)
}

"test.rxFeaturize.text.charFeatureExtractor.ngramCount" <- function()
{
    #library(MicrosoftML)
    trainReviews <- data.frame(opinion = c(
        "I love it",
        "I love it",
        "Love it",
        "I love it a lot",
        "Really love it",
        "I love it",
        "I love it",
        "love it",
        "I love it a lot",
        "Really love it",
        "I hate it",
        "I hate it",
        "I hate it",
        "hate it",
        "hate and",
        "I hate it and",
        "I hate it and",
        "I hate it and",
        "hate it and",
        "hate and"),
        like = rep(c(TRUE, FALSE), each = 10),
        stringsAsFactors = FALSE)

    #NOTE (lgong) 2017-04-04: the default "auto" behaves differently for different data input
    normalize <- "warn" #"auto" #"no"   #"yes"  #

    ## train directly with mlTransforms
    outModel.charcount1 <- rxLogisticRegression(like ~ features, data = trainReviews,
        l2Weight = 1, l1Weight = 1, optTol = 1e-07, memorySize = 20,
        initWtsScale = 0, maxIterations = 2147483647, showTrainingStats = FALSE,
        sgdInitTol = 0, trainThreads = NULL, denseOptimizer = FALSE, 
        normalize = normalize, 
        mlTransforms = list(featurizeText(
            vars = list(features = "opinion")
            , language = "English"
            , stopwordsRemover = NULL   #ignored
            , case = "lower"
            , keepDiacritics = FALSE
            , keepPunctuations = TRUE
            , keepNumbers = TRUE
            , dictionary = NULL         #ignored
            , wordFeatureExtractor = NULL
            , charFeatureExtractor = ngramCount()
            , vectorNormalizer = "l2"
           )
        )
    )
    coef(outModel.charcount1)
    #MicrosoftML:::print.maml(outModel.charcount1$mamlCode)

    ## featurize first and then train
    ftext.charcount <- rxFeaturize(data = trainReviews,
        dataThreads = NULL, randomSeed = NULL, maxSlots = 5000, 
        mlTransforms = list(featurizeText(
            vars = list(features = "opinion")
            , language = "English"
            , stopwordsRemover = NULL   #ignored
            , case = "lower"
            , keepDiacritics = FALSE
            , keepPunctuations = TRUE
            , keepNumbers = TRUE
            , dictionary = NULL         #ignored
            , wordFeatureExtractor = NULL
            , charFeatureExtractor = ngramCount()
            , vectorNormalizer = "l2"
            )
        )
    )

    #TODO (lgong) 2017-04-04: feature names contain symbols for start/end of text, which are not syntactically valid names.
    allvars <- colnames(ftext.charcount)
    allvars <- make.names(allvars, unique = TRUE)   #Make syntactically valid names out of character vectors. 
    colnames(ftext.charcount) <- allvars
    yvar <- "like"
    tvar <- "opinion"
    xvars <- setdiff(allvars, c(yvar, tvar))
    form <- paste(yvar, paste(xvars, collapse = " + "), sep  = " ~ ")

    outModel.charcount2 <- rxLogisticRegression(form, data = ftext.charcount,
        l2Weight = 1, l1Weight = 1, optTol = 1e-07, memorySize = 20,
        initWtsScale = 0, maxIterations = 2147483647, showTrainingStats = FALSE,
        sgdInitTol = 0, trainThreads = NULL, denseOptimizer = FALSE, 
        normalize = normalize
    )
    coef(outModel.charcount2)
    #MicrosoftML:::print.maml(outModel.charcount2$mamlCode)

    ## check
    #checkEquals(coef(outModel.charcount1), coef(outModel.charcount2), checkNames = FALSE)
}

"test.rxFeaturize.text.charFeatureExtractor.ngramHash" <- function()
{
    #library(MicrosoftML)
    trainReviews <- data.frame(opinion = c(
        "I love it",
        "I love it",
        "Love it",
        "I love it a lot",
        "Really love it",
        "I love it",
        "I love it",
        "love it",
        "I love it a lot",
        "Really love it",
        "I hate it",
        "I hate it",
        "I hate it",
        "hate it",
        "hate and",
        "I hate it and",
        "I hate it and",
        "I hate it and",
        "hate it and",
        "hate and"),
        like = rep(c(TRUE, FALSE), each = 10),
        stringsAsFactors = FALSE)

    #NOTE (lgong) 2017-04-04: the default "auto" behaves differently for different data input
    normalize <- "warn" #"auto" #"no"   #"yes"  #

    #TODO (lgong) 2017-04-04: why the results are different when hashBits is greater than 12?
    hashBits <- 12
    seed <- 314489979

    ## train directly with mlTransforms
    outModel.charhash1 <- rxLogisticRegression(like ~ features, data = trainReviews,
        l2Weight = 1, l1Weight = 1, optTol = 1e-07, memorySize = 20,
        initWtsScale = 0, maxIterations = 2147483647, showTrainingStats = FALSE,
        sgdInitTol = 0, trainThreads = NULL, denseOptimizer = FALSE, 
        normalize = normalize, 
        mlTransforms = list(featurizeText(
            vars = list(features = "opinion")
            , language = "English"
            , stopwordsRemover = NULL   #ignored
            , case = "lower"
            , keepDiacritics = FALSE
            , keepPunctuations = TRUE
            , keepNumbers = TRUE
            , dictionary = NULL         #ignored
            , wordFeatureExtractor = NULL
            , charFeatureExtractor = ngramHash(hashBits = hashBits, seed = seed)
            , vectorNormalizer = "l2"
           )
        )
    )
    coef(outModel.charhash1)
    #MicrosoftML:::print.maml(outModel.charhash1$mamlCode)

    ## featurize first and then train
    ftext.charhash <- rxFeaturize(data = trainReviews,
        dataThreads = NULL, randomSeed = NULL, maxSlots = 5000, 
        mlTransforms = list(featurizeText(
            vars = list(features = "opinion")
            , language = "English"
            , stopwordsRemover = NULL   #ignored
            , case = "lower"
            , keepDiacritics = FALSE
            , keepPunctuations = TRUE
            , keepNumbers = TRUE
            , dictionary = NULL         #ignored
            , wordFeatureExtractor = NULL
            , charFeatureExtractor = ngramHash(hashBits = hashBits, seed = seed)
            , vectorNormalizer = "l2"
            )
        )
    )

    allvars <- colnames(ftext.charhash)
    yvar <- "like"
    tvar <- "opinion"
    xvars <- setdiff(allvars, c(yvar, tvar))
    form <- paste(yvar, paste(xvars, collapse = " + "), sep  = " ~ ")

    outModel.charhash2 <- rxLogisticRegression(form, data = ftext.charhash,
        l2Weight = 1, l1Weight = 1, optTol = 1e-07, memorySize = 20,
        initWtsScale = 0, maxIterations = 2147483647, showTrainingStats = FALSE,
        sgdInitTol = 0, trainThreads = NULL, denseOptimizer = FALSE, 
        normalize = normalize
    )
    coef(outModel.charhash2)
    #MicrosoftML:::print.maml(outModel.charhash2$mamlCode)

    ## check
    #checkEquals(coef(outModel.charhash1), coef(outModel.charhash2), checkNames = FALSE, tolerance = 1e-5)
}
